#<profile4xi profile = "03" /> 
# -------------------------------------------------------------------------------------------------
#<program4xi program = "syncmanager4xi.py">
#<abstract4xi abstract = "provides class SyncManager4Xi"/>
#<refer4xi reference = "managers.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'SyncManager4Xi' ] 
#"
#/>

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
try:                 # system ones
    import os
    import sys
    import signal
except:
    pass
try:                 # threading ones       
    import threading                
    import _thread   
except:
    pass
try:
    import _winapi
except:
    pass

                
try:                 # most-common ones
    import weakref
    import array
    from traceback import format_exc
    import copyreg         
except:
    pass
try:                 # rare ones
    import queue
    import itertools          
except:
    pass 
try:                 # imported-as ones 
    pass          
except:
    pass            
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
try:                             # Binary ones
    import _multiprocessing
except:
    pass
try:                             # Python ones
    from multiprocessing.managers import SyncManager
except:
    print("syncmanager4xi.py *** failed in from multiprocessing.managers import SyncManager ***")
try:                             
    from multiprocessing.managers import BaseManager
except:
    print("syncmanager4xi.py *** failed in from multiprocessing.managers import BaseManager ***")    
try:                             
    from multiprocessing.managers import AutoProxy
except:
    print("syncmanager4xi.py *** failed in from multiprocessing.managers import AutoProxy ***")  
#"
#/>


# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi
from acquirerproxy4xi import AcquirerProxy4Xi

from after_fork4xi import AfterFork4Xi

from active_children4xi import active_children4xi
from answer_challenge4xi import answer_challenge4xi

from arbitrary_address4xi import arbitrary_address4xi
from array4xi import Array4Xi
from arrayproxy4xi import ArrayProxy4Xi


from autoproxy4xi import AutoProxy4Xi

from barrier4xi import Barrier4Xi
from barrierproxy4xi import BarrierProxy4Xi
from basemanager4xi import BaseManager4Xi 
from baseproxy4xi import BaseProxy4Xi


from current_process4xi import CurrentProcess4Xi
from current_process4xi import get_authkey_current_process4xi

from conditionproxy4xi import ConditionProxy4Xi
from constant_util4xi import ConstantUtil4Xi

from constant_connection4xi import ConstantConnection4Xi
from constant_family4xi import ConstantFamily4Xi
from constant_forking4xi import ConstantForking4Xi
from constant_heap4xi import ConstantHeap4Xi
from constant_logger4xi import ConstantLogger4Xi
from constant_managers4xi import ConstantManagers4Xi
from constant_pool4xi import ConstantPool4Xi
from constant_process4xi import ConstantProcess4Xi
from constant_synchronize4xi import ConstantSynchronize4Xi
from constant_system4xi import ConstantSystem4Xi
from constant_util4xi import ConstantUtil4Xi

from deliver_challenge4xi import deliver_challenge4xi
from dictproxy4xi import DictProxy4Xi
from dispatch4xi import dispatch4xi

from eventproxy4xi import EventProxy4Xi
from examine4xi import examine4xi

from finalize4xi import Finalize4Xi                 # use directly its registry and counter in the same way as Finalize4Xi class

from iteratorproxy4xi import IteratorProxy4Xi


from listener_client4xi import listener_client4xi
from listproxy4xi import ListProxy4Xi
from loop4xi import limit2loop4xi

from namespaceproxy4xi import NamespaceProxy4Xi
from namespace4xi import Namespace4Xi
from namespaceproxy4xi import NamespaceProxy4Xi

from sleep4xi import omega2sleep4xi 

from pipe4xi import Pipe4Xi
from poolproxy4xi import PoolProxy4Xi
from pool4xi import Pool4Xi
from process4xi import Process4Xi

from reason_object4xi import reason_object4xi

from sense_object4xi import sense_object4xi
from server4xi import Server4Xi
from sleep4xi import omega2sleep4xi
from state4xi import State4Xi

from timeout4xi import omega2timeout4xi
from token4xi import Token4Xi

from value4xi import Value4Xi
from valueproxy4xi import ValueProxy4Xi

from _run_finalizers4xi import _run_finalizers4xi
from timeout4xi import omega2timeout4xi

from print4xi import print4xi
from time4xi import time4xi    
#"
#/>




#
# =================================================================================================
#<class4xi class = "SyncManager4Xi" base = "BaseManager4Xi" child = "None" synopsis = "
#   Supports a number of shared object types.
#   The types registered are those intended for the synchronization of threads, plus dict, list and Namespace.
#   SyncManager4Xi is inherited from BaseManager4Xi.
#
#   This module TAILORS all the methods of BaseManager
#"
#
# usage = "
#    __init__.py
#        Manager: <--- gets called when a new Manager object is created by an application
#            from multiprocessing.managers import SyncManager
#            m = SyncManager()
#            m.start()
#
#    manager4xi.py: 
#        m2xi = SyncManager4Xi()
#    __init__.py:
#        m4xi = SyncManager4Xi()
#"
#>
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/> 
# =================================================================================================
class SyncManager4Xi(object):
    '''
    SyncManager4Xi class supports a number of shared object types.
    The types registered are those intended for the synchronization of threads, plus `dict`, `list` and `Namespace`.
    SyncManager is inherited from BaseManager in multiprocessing package with no methods.
    This module, however, tailors the methods used by BaseManager4Xi and inherits directly from object !!!!!!!!!!!!!!     
    '''
    
    
    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi expose = "
    _exposed_ = ('register', 'join',                 
                 'start', 'connect', 'get_server',
                 'sense', 'reason', 'ace')          
    #"
    #/>
           
    
    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    _registry = {}                             # registry of SyncManager
                        
    _Server = Server4Xi                        # server used
    
    _afterfork2xi = AfterFork4Xi()             # dragonized object
    
    
    _connectionc2xi = ConstantConnection4Xi()  # dragonized constant objects   
    _familyc2xi = ConstantFamily4Xi()
    _forkingc2xi = ConstantForking4Xi()
    _heapc2xi = ConstantHeap4Xi()
    _loggerc2xi = ConstantLogger4Xi()
    _managersc2xi = ConstantManagers4Xi() 
    _poolc2xi = ConstantPool4Xi()
    _processc2xi = ConstantProcess4Xi()
    _synchronizec2xi = ConstantSynchronize4Xi()
    _systemc2xi = ConstantSystem4Xi()
    _utilc2xi = ConstantUtil4Xi()

  
    _ack2connection = True                     # dragonized states
    _nack2connection = False
    
    _ack2server = True
    _nack2server = False
    
    _ack2dispatch = False
    _nack2dispatch = True    

    _ack2shutdown = 0                          # dragonized statistics
    _nack2shutdown = 0
    
    _ack2type = 0                          
    _nack2type = 0
    #"
    #/>
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-SyncManager4Xi" usage = "initializes SyncManager4Xi">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>                            
    #   NACK4PX: Step 7/14
    # ---------------------------------------------------------------------------------------------
    def __init__(self, hub=None, passport=None, force=None):
        '''
        initializes SyncManager4Xi
        '''

       
        # -----------------------------------------------------------------------------------------
        # (Step 1/14) - ACK4PX
        # -----------------------------------------------------------------------------------------
        class2print = "SyncManager4Xi"
        method2print = "__init__"
        
        message2print = "{(Step 1/14a) started" + "; init-of-SyncManager4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
                        
        message2print = "{(Step 1/14b) borning" + "; syncmanager2xi-of-SyncManager4Xi-borning: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        #PX: 
        
        message2print = "{(Step 1/14c) inheriting" + "; object-inheriting-by-syncmanager2xi-of-SyncManager4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        #PX: <class 'object'>               
                
        if hub is None:
            try:
                family2sync = SyncManager4Xi._familyc2xi.get_default_family()               
                
                message2print = "{(Step 1/14d1) set" + "; family2sync-set-by-init-of-SyncManager4Xi to: " + str(family2sync)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                # 
                
                hub = arbitrary_address4xi(family2sync) 
                
                message2print = "{(Step 1/14d2) set" + "; hub-set-by-init-of-SyncManager4Xi to: " + str(hub)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                # 
            except:
                message2print = "{(Step 1/14dz) warned" + "; hub-warned-by-init-of-SyncManager4Xi: " + str(hub) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                  
                # 
        else:
            message2print = "{(Step 1/14d2) passed" + "; hub-passed-to-init-of-SyncManager4Xi: " + str(hub) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                  
            #PX: \\.\pipe\pyc-4248-0-7aso9i
            
        if passport is None:
            message2print = "{(Step 1/14e1) warned" + "; passport-warned-by-init-of-SyncManager4Xi: " + str(passport)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
        
            passport = os.urandom(32)

            message2print = "{(Step 1/14e2) set" + "; passport-set-by-init-of-SyncManager4Xi to: " + str(passport)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
            #PX: None
        else:
            message2print = "{(Step 1/14e3) passed" + "; passport-passed-to-init-of-SyncManager4Xi: " + str(passport) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                  
            #PX: b'\xc3\xfb\r\x9e\xed\xe5sQ\x8c\x01\xb7\x9c\xc7\xe8\x9a\xe9G\x82\x19\x8e\xa4Y\xd2E\x9c\x0e\xa1%\xfc&\xd6@'
        
                                
        if force is None:
            force='pickle'
            
            message2print = "{(Step 1/14f1) set" + "; force-set-by-init-of-SyncManager4Xi to: " + str(force)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
        else:                
            message2print = "{(Step 1/14f2) passed" + "; force-passed-to-init-of-SyncManager4Xi: " + str(force) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                  
            #PX: pickle 

        
        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 
            force = 'pickle'
            
            message2print = "{(StepOriginal) set" + "; force-set-by-init-of-SyncManager4Xi to: " + str(force)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
  
            message2print = "{(StepOriginal) passing" + "; hub-passing-to-SyncManager-init-by-init-of-SyncManager4Xi: " + str(hub) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                            
            
            message2print = "{(StepOriginal) passing" + "; passport-passing-to-SyncManager-init-by-init-of-SyncManager4Xi: " + str(passport) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            
            message2print = "{(StepOriginal) passing" + "; force-passing-to-SyncManager-init-by-init-of-SyncManager4Xi: " + str(force) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                                  
            
            SyncManager.__init__(self, hub, passport, force)    # note serialize parameter setting
            
            message2print = "{(StepOriginal) performed" + "; init-of-SyncManager-performed-by-init-of-SyncManager4Xi to: " + str(SyncManager.__init__)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
        else:
                
          
            # -----------------------------------------------------------------------------------------
            # (Step 2/14) -  
            # ----------------------------------------------------------------------------------------
            
            message2print = "{(Step 2/14a) commented" + "; avoiding-call-commented-by-init-of-SyncManager4Xi: * did NOT call base class BaseManager4Xi __init__ method * " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            
            
            Finalize4Xi.__registry = {}                                  #<--- dragonized 

            message2print = "{(Step 2/14b) initialized" + "; finalizer-registry-initialized by-init-of-SyncManager4Xi to: " + str(Finalize4Xi.__registry)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #PX: {}                                    
            
            Finalize4Xi._counter = itertools.count()                     #<--- dragonized 
    
            message2print = "{(Step 2/14c) performed" + "; init-finalizer-counter4xi-performed-by-init-of-SyncManager4Xi: " + str(Finalize4Xi._counter) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            #PX: count(0)                
                            
            
            # =========================================================================================
            # (Step 3/14) - SHARED ONES
            # =========================================================================================
 
            # -----------------------------------------------------------------------------------------
            # (Step 3/14a)
            # -----------------------------------------------------------------------------------------
    
            message2print = "{(Step 3/14a) sharing" + "; afterfork2xi-sharing-by-syncmanager2xi-of-SyncManager4Xi: " + str(SyncManager4Xi._afterfork2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                      
            #PX: <after_fork4xi.AfterFork4Xi object at 0x02E2AFD0>
            
            # -----------------------------------------------------------------------------------------
            # (Step 3/14b)
            # -----------------------------------------------------------------------------------------

            message2print = "{(Step 3/14b) sharing" + "; managersc2xi-sharing-by-syncmanager2xi-of-SyncManager4Xi: " + str(SyncManager4Xi._managersc2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                      
            #PX: <constant_managers4xi.ConstantManagers4Xi object at 0x02E2EA50> 
             
            # -----------------------------------------------------------------------------------------
            # (Step 3/14c)
            # -----------------------------------------------------------------------------------------

            message2print = "{(Step 3/14c) sharing" + "; utilc2xi-sharing-by-syncmanager2xi-of-SyncManager4Xi: " + str(SyncManager4Xi._utilc2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                      
            #PX: <constant_util4xi.ConstantUtil4Xi object at 0x02E2EB90> 
              
            # -----------------------------------------------------------------------------------------
            # (Step 3/14d)
            # -----------------------------------------------------------------------------------------

            message2print = "{(Step 3/14d) sharing" + "; connectionc2xi-sharing-by-syncmanager2xi-of-SyncManager4Xi: " + str(SyncManager4Xi._connectionc2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                      
            #PX: <constant_connection4xi.ConstantConnection4Xi object at 0x02E2AFF0> 
              
            # -----------------------------------------------------------------------------------------
            # (Step 3/14e)
            # -----------------------------------------------------------------------------------------

            message2print = "{(Step 3/14e) sharing" + "; familyc2xi-sharing-by-syncmanager2xi-of-SyncManager4Xi: " + str(SyncManager4Xi._familyc2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                      
            #PX: <constant_family4xi.ConstantFamily4Xi object at 0x02E2E050> 
              
            # -----------------------------------------------------------------------------------------
            # (Step 3/14f)
            # -----------------------------------------------------------------------------------------

            message2print = "{(Step 3/14f) sharing" + "; forkingc2xi-sharing-by-syncmanager2xi-of-SyncManager4Xi: " + str(SyncManager4Xi._forkingc2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                      
            #PX: <constant_forking4xi.ConstantForking4Xi object at 0x02E2E9F0> 
              
            # -----------------------------------------------------------------------------------------
            # (Step 3/14g)
            # -----------------------------------------------------------------------------------------
           
            message2print = "{(Step 3/14g) sharing" + "; heapc2xi-sharing-by-syncmanager2xi-of-SyncManager4Xi: " + str(SyncManager4Xi._heapc2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                      
            #PX: <constant_heap4xi.ConstantHeap4Xi object at 0x02E2EA10>  
              
            # -----------------------------------------------------------------------------------------
            # (Step 3/14h)
            # -----------------------------------------------------------------------------------------

            message2print = "{(Step 3/14h) sharing" + "; loggerc2xi-sharing-by-syncmanager2xi-of-SyncManager4Xi: " + str(SyncManager4Xi._loggerc2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                      
            #PX: <constant_logger4xi.ConstantLogger4Xi object at 0x02E2EA30> 
              
            # -----------------------------------------------------------------------------------------
            # Step 3/14i) 
            # -----------------------------------------------------------------------------------------

            message2print = "{(Step 3/14i) sharing" + "; poolc2xi-sharing-by-syncmanager2xi-of-SyncManager4Xi: " + str(SyncManager4Xi._poolc2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                      
            #PX: <constant_pool4xi.ConstantPool4Xi object at 0x02E2EA90> 
              
            # -----------------------------------------------------------------------------------------
            # (Step 3/14j)
            # -----------------------------------------------------------------------------------------

            message2print = "{(Step 3/14j) sharing" + "; processc2xi-sharing-by-syncmanager2xi-of-SyncManager4Xi: " + str(SyncManager4Xi._processc2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                      
            #PX: <constant_process4xi.ConstantProcess4Xi object at 0x02E2EAF0>  
              
            # -----------------------------------------------------------------------------------------
            # (Step 3/14k)
            # -----------------------------------------------------------------------------------------

            message2print = "{(Step 3/14k) sharing" + "; synchronizec2xi-sharing-by-syncmanager2xi-of-SyncManager4Xi: " + str(SyncManager4Xi._synchronizec2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                      
            #PX: <constant_synchronize4xi.ConstantSynchronize4Xi object at 0x02E2EC30> 
              
            # -----------------------------------------------------------------------------------------
            # (Step 3/14l)
            # -----------------------------------------------------------------------------------------

            message2print = "{(Step 3/14l) sharing" + "; systemc2xi-sharing-by-syncmanager2xi-of-SyncManager4Xi: " + str(SyncManager4Xi._systemc2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                      
            #PX: <constant_system4xi.ConstantSystem4Xi object at 0x02E2EB70> 
 
            
            # -----------------------------------------------------------------------------------------
            # (Step 4/14) - ACK4PX      
            #
            # =========================================================================================
            # SyncManager4Xi.register parameters: (cls, typeid, callable=None, proxytype=None, exposed=None, method_to_typeid=None, create_method=True)
            # 1) cls <-- class, for example Queue4Xi
            # 2) typeid <-- type, for example queue.Queue; cls._registry[typeid] <-- KEY to cls._registry
            # 3) callable <-- optional
            # 4) proxytype <-- optional, for example EventProxy4Xi, if not given set to AutoProxy4Xi
            # 5) exposed <-- optional
            # 6) method_to_typeid <-- optional
            # 7) create_method <-- optional; example: False with Iterataror4Xi 
            # cls._registry[typeid] = (callable, exposed, method_to_typeid, proxytype) i.e. typeid act as the key
            # =========================================================================================
    
            #a)
            # -----------------------------------------------------------------------------------------
            SyncManager4Xi.register('Queue4Xi', queue.Queue) #original SyncManager.register('Queue', queue.Queue) where queue imported from standard Python lib and Queue class is queues.py module
           
            # cls: <class 'syncmanager4xi.SyncManager4Xi'>; typeid: Queue4Xi; callable: <class 'queue.Queue'>
            
            #b) 
            # -----------------------------------------------------------------------------------------        
            SyncManager4Xi.register('JoinableQueue4Xi', queue.Queue)
       
            # JoinableQueue4Xi registered as queue.Queue
            
            #c) 
            # -----------------------------------------------------------------------------------------        
            SyncManager4Xi.register('Event4Xi', threading.Event, EventProxy4Xi)
         
            # Event4Xi registered as threading.Event with EventProxy4Xi
            
            #d) 
            # -----------------------------------------------------------------------------------------        
            SyncManager4Xi.register('Lock4Xi', threading.Lock, AcquirerProxy4Xi)
         
            # Lock4Xi registered as threading.Lock with  AcquirerProxy4Xi
            
            #e) 
            # -----------------------------------------------------------------------------------------
            SyncManager4Xi.register('RLock4Xi', threading.RLock, AcquirerProxy4Xi)
         
            # RLock4Xi registered as threading.RLock with AcquirerProxy4Xi
            
            #f) 
            # -----------------------------------------------------------------------------------------        
            SyncManager4Xi.register('Semaphore4Xi', threading.Semaphore, AcquirerProxy4Xi)
          
            # Semaphore4Xi registered as threading.Semaphore with AcquirerProxy4Xi
            
            #g) 
            # -----------------------------------------------------------------------------------------        
            SyncManager4Xi.register('BoundedSemaphore4Xi', threading.BoundedSemaphore, AcquirerProxy4Xi)
         
            # BoundedSemaphore4Xi registered as threading.BoundedSemaphore with AcquirerProxy4Xi
            
            #h) 
            # -----------------------------------------------------------------------------------------        
            SyncManager4Xi.register('Condition4Xi', threading.Condition, ConditionProxy4Xi)
          
            # Condition4Xi registered as threading.Condition with ConditionProxy4Xi
            
            #i) 
            # -----------------------------------------------------------------------------------------        
            SyncManager4Xi.register('Pool4Xi', Pool4Xi, PoolProxy4Xi)
          
            # Pool4Xi registered as Pool4Xi with PoolProxy4Xi
            
            #j) 
            # -----------------------------------------------------------------------------------------        
            SyncManager4Xi.register('list', list, ListProxy4Xi)
         
            # list registered as list with ListProxy4Xi
            
            #k) 
            # -----------------------------------------------------------------------------------------        
            SyncManager4Xi.register('dict', dict, DictProxy4Xi)
          
            # dict registered as dict with DictProxy4Xi
            
            #l) 
            # -----------------------------------------------------------------------------------------        
            SyncManager4Xi.register('Value4Xi', Value4Xi, ValueProxy4Xi)
         
            # Value4Xi registered as Value4Xi with ValueProxy4Xi
            
            #m) 
            # -----------------------------------------------------------------------------------------        
            SyncManager4Xi.register('Array4Xi', Array4Xi, ArrayProxy4Xi)
          
            # Array4Xi registered as Array4Xi with ArrayProxy4Xi 
         
           
            
            #n) 
            # -----------------------------------------------------------------------------------------        
            SyncManager4Xi.register('Namespace4Xi', Namespace4Xi, NamespaceProxy4Xi)
          
            # Namespace4Xi registered as Namespace4Xi with NamespaceProxy4Xi
    
            #o) 
            # -----------------------------------------------------------------------------------------        
            SyncManager4Xi.register('Iterator', proxytype=IteratorProxy4Xi, create_method=False)
          
            # Iterator registered with IteratorProxy4Xi, create_method=False
            
            #p) 
            # -----------------------------------------------------------------------------------------        
            SyncManager4Xi.register('AsyncResult', create_method=False)        
         
            #q) 
            # -----------------------------------------------------------------------------------------        
            SyncManager4Xi.register('Barrier4Xi', threading.Barrier, BarrierProxy4Xi) 
            
            
            # AsyncResult registered with create_method=False
            
            
            # -------------------------------------------------------------------------------------------------
            # managers.py code as comparison    
            # -------------------------------------------------------------------------------------------------
            #01) SyncManager.register('Queue', queue.Queue)
            #02) SyncManager.register('JoinableQueue', queue.Queue)
            #03) SyncManager.register('Event', threading.Event, EventProxy)
            #04) SyncManager.register('Lock', threading.Lock, AcquirerProxy)
            #05) SyncManager.register('RLock', threading.RLock, AcquirerProxy)
            #06) SyncManager.register('Semaphore', threading.Semaphore, AcquirerProxy)
            #07) SyncManager.register('BoundedSemaphore', threading.BoundedSemaphore, AcquirerProxy)
            #08) SyncManager.register('Condition', threading.Condition, ConditionProxy)
            #09) SyncManager.register('Pool', Pool, PoolProxy)
            #10) SyncManager.register('list', list, ListProxy)
            #11) SyncManager.register('dict', dict, DictProxy)
            #12) SyncManager.register('Value', Value, ValueProxy)
            #13) SyncManager.register('Array', Array, ArrayProxy)
            #14) SyncManager.register('Namespace', Namespace, NamespaceProxy)
            #15) SyncManager.register('Iterator', proxytype=IteratorProxy, create_method=False)
            #16) SyncManager.register('AsyncResult', create_method=False)
            
      
     
                   
            # -----------------------------------------------------------------------------------------
            # (Step 5/14) - ACK4PX
            # -----------------------------------------------------------------------------------------
            if passport is None: 

                passport = get_authkey_current_process4xi()
                
                message2print = "{(Step 5/14a) set" + "; passport-set-by-init-of-SyncManager4Xi to: " + str(passport)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                #PX: b'\x08\xf1\xf7\x04\x90\xf9P\xcc\xa4e\x1b\x89\x06o\x0b\x84K\xb5ml\xaf\xf9\xe9\xaf\x97\x18\x8b\x91\xf5\x93\x1b"'

                
            
            # -----------------------------------------------------------------------------------------    
            # (Step 6/14) - removed
            # -----------------------------------------------------------------------------------------                 

                
                  
            # -----------------------------------------------------------------------------------------
            # (Step 7/14) - NACK4PX    
            # -----------------------------------------------------------------------------------------    
            self._address = hub  
            
            message2print = "{(Step 7/14) set" + "; hub-of-syncmanager2xi-set-by-init-of-SyncManager4Xi to: " + str(self._address)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            #PX: \\.\pipe\pyc-4248-0-7aso9i 
                          
            # -----------------------------------------------------------------------------------------    
            # (Step 8/14) - ACK4PX
            # -----------------------------------------------------------------------------------------         
            self._authkey = passport                # <--- dragonized
            
            message2print = "{(Step 8/14) set" + "; passport-of-syncmanager2xi-set-by-init-of-SyncManager4Xi to: " + str(self._authkey)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            #PX: b'\xc3\xfb\r\x9e\xed\xe5sQ\x8c\x01\xb7\x9c\xc7\xe8\x9a\xe9G\x82\x19\x8e\xa4Y\xd2E\x9c\x0e\xa1%\xfc&\xd6@'
            
            # -----------------------------------------------------------------------------------------    
            # (Step 9/14) - ACK4PX
            # -----------------------------------------------------------------------------------------         
            self._state = State4Xi() 
            
            message2print = "{(Step 9/14) set" + "; state-of-syncmanager2xi-set-by-init-of-SyncManager4Xi to: " + str(self._state)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            #PX: <state4xi.State4Xi object at 0x033BAF90>   

            # -----------------------------------------------------------------------------------------    
            # (Step 10/14) - ACK4PX
            # -----------------------------------------------------------------------------------------         
            self._state.value = State4Xi.INITIAL
            
            message2print = "{(Step 10/14) set" + "; state-value-of-syncmanager2xi-set-by-init-of-SyncManager4Xi to: " + str(self._state.value)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            #PX: 0
            
            # -----------------------------------------------------------------------------------------    
            # (Step 11/14) - ACK4PX
            # -----------------------------------------------------------------------------------------         
            self._serializer = force 
            
            message2print = "{(Step 11/14) set" + "; force-of-syncmanager2xi-set-by-init-of-SyncManager4Xi to: " + str(self._serializer)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            #PX: pickle   

            
            
            # -----------------------------------------------------------------------------------------    
            # (Step 12/14) - ACK4PX
            # -----------------------------------------------------------------------------------------         
            self._Listener, self._Client = listener_client4xi[force]      
            
            message2print = "{(Step 12/14a) set" + "; Listener-of-syncmanager2xi-set-by-init-of-SyncManager4Xi to: " + str(self._Listener)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            #PX: <class 'listener4xi.Listener4Xi'>
            
            message2print = "{(Step 12/14b) set" + "; Client-of-syncmanager2xi-set-by-init-of-SyncManager4Xi to: " + str(self._Client)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            #PX: <function Client4Xi at 0x02E8D738>
            
            # -----------------------------------------------------------------------------------------    
            # (Step 13/14) - ACK4PX
            # -----------------------------------------------------------------------------------------         

            message2print = "{(Step 13/14a) inherited" + "; BaseManager4Xi-inherited-by-syncmanager2xi-of-SyncManager4Xi: " + str(BaseManager4Xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
            #PX: <class 'basemanager4xi.BaseManager4Xi'>
                 
            message2print = "{(Step 13/14b) activated" + "; syncmanager2xi-of-SyncManager4Xi-activated: " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
            #PX: OK
            
            # -----------------------------------------------------------------------------------------
            # (Step 14/14) - NACK4PX (see above)    
            # -----------------------------------------------------------------------------------------
            #PX: OK
            
            message2print = "{(Step 14/14a) sleeping" + "; syncmanager2xi-of-SyncManager4Xi-sleeping-by-init-of-SyncManager4Xi: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                      
            
            omega2sleep4xi(class2print, method2print)                
              
            message2print = "{(Step 14/14b) returning" + "; init-of-SyncManager4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reduce-of-SyncManager4Xi" usage = "reduces"> 
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>                               
    # ---------------------------------------------------------------------------------------------
    def __reduce__(self):
        '''
        reduces 
        '''
        
        # -----------------------------------------------------------------------------------------    
        # PX: not called 
        # -----------------------------------------------------------------------------------------          
        class2print = "SyncManager4Xi"
        method2print = "__reduce__"
        
        message2print = "{started" + "; reduce-of-SyncManager4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
        # -----------------------------------------------------------------------------------------    
        #
        # ----------------------------------------------------------------------------------------- 
                        
        type2xi = None
        
        try:
        
            # -------------------------------------------------------------------------------------    
            #
            # ------------------------------------------------------------------------------------- 
            
            type2xi = type(self).from_address, \
                   (self._address, self._authkey, self._serializer)
                   
            SyncManager4Xi._ack2type = SyncManager4Xi._ack2type + 1 
                         
        except:
        
            # -------------------------------------------------------------------------------------    
            #
            # ------------------------------------------------------------------------------------- 
            
            message2print = "{yielded" + "; type2xi-yielded-by-reduce-of-SyncManager4Xi: " + str(" *** ") + str(type2xi) + str(" *** ") + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 

            SyncManager4Xi._nack2type = SyncManager4Xi._nack2type + 1    
        
        # -----------------------------------------------------------------------------------------    
        #
        # ----------------------------------------------------------------------------------------- 
                                  
        message2print = "{returning" + "; type2xi-returning-by-reduce-of-SyncManager4Xi: " + str(type2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                                               
               
        return type2xi
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get-server-of-SyncManager4Xi" usage = "returns server object with serve_forever() method and address attribute">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>                                
    # ---------------------------------------------------------------------------------------------
    def get_server(self):
        '''
        returns server object with serve_forever() method and address attribute
        '''

        # -----------------------------------------------------------------------------------------    
        # PX: not called! 
        # -----------------------------------------------------------------------------------------                 
           
        class2print = "SyncManager4Xi"
        method2print = "get_server" 
        
        message2print = "{started" + "; get-server-of-SyncManager4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
   
        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 
            
            message2print = "{(StepOriginal) calling" + "; SyncManager-get-server-calling-by-get-server-of-SyncManager4Xi: " + str(SyncManager.get_server) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
   
            return SyncManager.get_server(self)
        else:    
                    
            # -----------------------------------------------------------------------------------------    
            #
            # -----------------------------------------------------------------------------------------                 

            if self._state.value == State4Xi.INITIAL:
            
                message2print = "{accepted" + "; state-value-of-syncmanager2xi-accepted-by-get-server-of-SyncManager4Xi: " + str(self._state.value) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)         
            else:
                message2print = "{yielded" + "; state-value-of-syncmanager2xi-yielded-by-get-server-of-SyncManager4Xi: " + str(self._state.value)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 

            # -----------------------------------------------------------------------------------------    
            #
            # -----------------------------------------------------------------------------------------                 
            
            message2print = "{passing" + "; registry-of-syncmanager2xi-passing-to-Server4Xi-by-get-server-of-SyncManager4Xi: " + str(self._registry) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            
            message2print = "{passing" + "; hub-of-syncmanager2xi-passing-to-Server4Xi-by-get-server-of-SyncManager4Xi: " + str(self._address) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            
            if self._address is None:
                message2print = "{yielded" + "; hub-of-syncmanager2xi-yielded-by-get-server-of-SyncManager4Xi: " + str(self._address)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
            
            
            message2print = "{passing" + "; passport-of-syncmanager2xi-passing-to-Server4Xi-by-get-server-of-SyncManager4Xi: " + str(self._authkey) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            
            message2print = "{passing" + "; force-of-syncmanager2xi-passing-to-Server4Xi-by-get-server-of-SyncManager4Xi: " + str(self._serializer) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                                     
            message2print = "{calling" + "; Server4Xi-calling-by-get-server-of-SyncManager4Xi: " + str(Server4Xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
                            

            # -----------------------------------------------------------------------------------------    
            #
            # -----------------------------------------------------------------------------------------                 
                                                     
            server2xi = None
            count2try = 0 
            
            SyncManager4Xi._ack2server = False
            SyncManager4Xi._nack2server = True
            
            max2count = limit2loop4xi(class2print, method2print) 
            message2print = "{got" + "; max2count-got-by-get-server-of-SyncManager4Xi: " + str(max2count)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
            
            while ((count2try < max2count) and  (SyncManager4Xi._ack2server is False)):
                try:
                    server2xi = Server4Xi(self._registry, self._address, self._authkey, self._serializer)
    
                    SyncManager4Xi._nack2server = False
                    SyncManager4Xi._ack2server = True
                    
                except:
                    message2print = "{yielded" + "; nack2server-yielded-by-get-server-of-SyncManager4Xi: " + str(SyncManager4Xi._nack2server)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 

                    count2try = count2try + 1

                                       
                    message2print = "{sleeping" + "; syncmanager2xi-of-SyncManager4Xi-sleeping-by-get-server-of-SyncManager4Xi: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                        
                    omega2sleep4xi(class2print, method2print)                                 

            # -----------------------------------------------------------------------------------------    
            #
            # -----------------------------------------------------------------------------------------                 
                                
            message2print = "{returning" + "; server2xi-returning-by-get-server-of-SyncManager4Xi: " + str(server2xi) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
                          
            return server2xi
    #</methodize4xi>
                       
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "connect-of-SyncManager4Xi" usage = "connects a manager object to the server process">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>                               
    # ---------------------------------------------------------------------------------------------
    def connect(self):
        '''
        connects a manager object to the server process
        '''

        #PX: not called !        
          
        # -----------------------------------------------------------------------------------------    
        # (Step 1/6)
        # -----------------------------------------------------------------------------------------         
        
        class2print = "SyncManager4Xi" 
        method2print = "connect"
        
        message2print = "{(Step 1/6a) started" + "; connect-of-SyncManager4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
     
        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 
            message2print = "{(StepOriginal) calling" + "; SyncManager-connect-calling-by-connect-of-SyncManager4Xi: " + str(SyncManager.connect) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
        
            SyncManager.connect(self)
             
        else:    
                    
                
            # -----------------------------------------------------------------------------------------    
            # (Step 2/6)
            # -----------------------------------------------------------------------------------------                 
            message2print = "{(Step 2/6a) passing" + "; force-of-syncmanager2xi-passing-to-listener-client4xi-by-connect-of-SyncManager4Xi: " + str(self._serializer) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                            
            Listener, Client = listener_client4xi[self._serializer]      #Listener4Xi and Client4Xi 
            
            message2print = "{(Step 2/6b) set" + "; Client-set-by-connect-of-SyncManager4Xi to: " + str(Client)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                            
            
            message2print = "{(Step 2/6c) set" + "; Listener-set-by-connect-of-SyncManager4Xi to: " + str(Listener)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                                 
            
            # -----------------------------------------------------------------------------------------    
            # (Step 3/6)
            # -----------------------------------------------------------------------------------------                 
            message2print = "{(Step 3/6a) passing" + "; hub-of-syncmanager2xi-passing-to-Client-by-connect-of-SyncManager4Xi: " + str(self._address) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            
            if self._address is None:
                message2print = "{(Step 3/6b) warned" + "; hub-of-syncmanager2xi-warned-by-connect-of-SyncManager4Xi: " + str(self._address)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
            
                self._address = arbitrary_address4xi(SyncManager4Xi._familyc2xi.get_default_family()) #dragonized forcing ! 
                
                message2print = "{(Step 3/6c) set" + "; hub-of-syncmanager2xi-set-by-connect-of-SyncManager4Xi to: " + str(self._address)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                
                 
                     
            message2print = "{(Step 3/6d) passing" + "; passport-of-syncmanager2xi-passing-to-Client-by-connect-of-SyncManager4Xi: " + str(self._authkey) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                          
                   
            SyncManager4Xi._ack2connection = False
            SyncManager4Xi._nack2connection = True
            
            max2count = limit2loop4xi(class2print, method2print) 
            message2print = "{(Step 3/6e) got" + "; max2count-got-by-connect-of-SyncManager4Xi: " + str(max2count)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            connection2xi = None
                         
            count2try = 0
            
            while ((count2try < max2count) and  (SyncManager4Xi._ack2connection is False)):
            
                try:
                    connection2xi = Client4Xi(self._address, SyncManager4Xi._familyc2xi.get_default_family(), self._authkey)
            
                    message2print = "{(Step 3/6f) set" + "; connection2xi-set-by-connect-of-SyncManager4Xi to: " + str(connection2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
                    
                    SyncManager4Xi._nack2connection = False
                    SyncManager4Xi._ack2connection = True
                except:
                            
                    count2try = count2try + 1
                    
                    message2print = "{(Step 3/6g) yielded" + "; nack2connection-yielded-by-connect-of-SyncManager4Xi: " + str(SyncManager4Xi._nack2connection)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    
                            
            
            # -----------------------------------------------------------------------------------------    
            # (Step 4/6)
            # -----------------------------------------------------------------------------------------                 
            SyncManager4Xi._ack2dispatch = False
            SyncManager4Xi._nack2dispatch = True
            
            if SyncManager4Xi._ack2connection is True:
                message2print = "{(Step 4/6a) passing" + "; connection2xi-passing-to-dispatch4xi-by-connect-of-SyncManager4Xi: " + str(connection2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                                
                message2print = "{(Step 4/6b) passing" + "; dummy-manager4xi-passing-to-dispatch4xi-by-connect-of-SyncManager4Xi: " + str(get_dummy_manager4xi()) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                 
                
                try:
                    dispatch4xi(connection2xi, None, get_dummy_manager4xi()) #DUMMY_MANAGER4XI)
                    
                    message2print = "{(Step 4/6c) performed" + "; dispatch4xi-performed-by-connect-of-SyncManager4Xi: " + str(dispatch4xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                
                    SyncManager4Xi._nack2dispatch = False
                    SyncManager4Xi._ack2dispatch = True
                    
                except:
                    message2print = "{(Step 4/6d) yielded" + "; nack2dispatch-yielded-by-connect-of-SyncManager4Xi: " + str(SyncManager4Xi._nack2dispatch)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                    
                             
            else:
                message2print = "{(Step 4/6e) yielded" + "; nack2connection-yielded-by-connect-of-SyncManager4Xi: " + str(SyncManager4Xi._nack2connection)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
            
                                
            
            # -----------------------------------------------------------------------------------------    
            # (Step 5/6)
            # -----------------------------------------------------------------------------------------                 
               
            self._state.value = State4Xi.STARTED
            
            message2print = "{(Step 5/6) set" + "; state-value-of-syncmanager2xi-set-by-connect-of-SyncManager4Xi to: " + str(self._state.value)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                            
                  
           
            
            # -----------------------------------------------------------------------------------------    
            # (Step 6/6)
            # -----------------------------------------------------------------------------------------                 
            message2print = "{(Step 6/6) returning" + "; connect-of-SyncManager4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                                    
            return                
    #</methodize4xi>
                                             

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "start-of-SyncManager4Xi" usage = "spawns a server process for this manager object">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>                                   
    # NACK4PX: (Step 4/12) passes None as address = *** FATAL ERROR *** to be solved !
    # ---------------------------------------------------------------------------------------------
    def start(self):
        ''' 
        spawns a server process for this manager object
        ''' 

        # -----------------------------------------------------------------------------------------    
        # (Step 0/12) - ACK4PX
        # -----------------------------------------------------------------------------------------      
        
        class2print = "SyncManager4Xi"
        method2print = "start"
        
        message2print = "{(Step 0/12a) started" + "; start-of-SyncManager4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #PX: 11:59:19 04/03/10 FLE Daylight Time
 
        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 

            message2print = "{(StepOriginal) calling" + "; SyncManager-start-calling-by-start-of-SyncManager4Xi: " + str(SyncManager.start) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
                            
       
            SyncManager.start(self)        
        else:
        
            # -----------------------------------------------------------------------------------------    
            # (Step 1/12) - ACK4PX
            # -----------------------------------------------------------------------------------------                 
            
            initial2xi = SyncManager4Xi._managersc2xi.get_initial()
            
            message2print = "{(Step 1/12) set" + "; initial2xi-set-by-start-of-SyncManager4Xi to: " + str(initial2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            #PX: 0
            
            
            # -----------------------------------------------------------------------------------------    
            # (Step 2/12)- ACK4PX
            # -----------------------------------------------------------------------------------------                 
               
            assert self._state.value == State4Xi.INITIAL
    
            message2print = "{(Step 2/12) asserted" + "; state-value-of-syncmanager2xi-asserted-by-start-of-SyncManager4Xi: " + str(self._state.value) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            #PX: 0
            
            
            # -----------------------------------------------------------------------------------------    
            # (Step 3/12)- ACK4PX
            # -----------------------------------------------------------------------------------------                 
                                                                            # PIPE OVER WHICH WE WILL RETRIEVE ADDRESS OF SERVER
            wave2read, wave2write = Pipe4Xi(duplex=False)                     
            
            message2print = "{(Step 3/12a) set" + "; wave2read-set-by-start-of-SyncManager4Xi to: " + str(wave2read)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            #PX: <read-only PipeConnection, handle 460>  
                                        
            
            message2print = "{(Step 3/12b) set" + "; wave2write-set-by-start-of-SyncManager4Xi to: " + str(wave2write)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            #PX: <write-only PipeConnection, handle 440>
            
            # -----------------------------------------------------------------------------------------    
            # (Step 4/12)- NACK4PX; Step 4/12d passes None as address - *** FATAL ERROR** to be solved!
            # -----------------------------------------------------------------------------------------                 
            message2print = "{(Step 4/12a) informing" + "; spawning-informing-by-start-of-SyncManager4Xi: " + " spawning a process which runs a server" + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #PX: spawning a process which runs a server 
            
            target2xi = type(self)._run_server  
     
            message2print = "{(Step 4/12b) using" + "; target-using-by-start-of-SyncManager4Xi to: " + str(target2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            #PX: <bound method type._run_server of <class 'syncmanager4xi.SyncManager4Xi'>>
            
            message2print = "{(Step 4/12b) informing" + "; target2xi-informing-by-start-of-SyncManager4Xi: " + " note: passed when Process4Xi is called target = " + str(target2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #PX: passed when Process4Xi is called target = 
            
            message2print = "{(Step 4/12c) passing" + "; registry-of-syncmanager2xi-in-args-passing-to-Process4Xi-by-start-of-SyncManager4Xi: " + str(self._registry) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            #PX: {    'Iterator4Xi': (None, ('send', 'throw', 'close', 'sense', 'reason', 'ace'), None, <class 'iteratorproxy4xi.IteratorProxy4Xi'>), 
            #         'AsyncResult': (None, None, None, <function AutoProxy4Xi at 0x0322E7C8>), 
            #        'Condition4Xi': (<function Condition at 0x02AD1108>, ('wait', 'notify', 'notify_all', 'sense', 'reason', 'ace'), None, <class 'conditionproxy4xi.ConditionProxy4Xi'>),
            #            'Event4Xi': (<function Event at 0x02AD1810>, ('is_set', 'set', 'clear', 'wait', 'sense', 'reason', 'ace'), None, <class 'eventproxy4xi.EventProxy4Xi'>),
            #            'Iterator': (None, ('send', 'throw', 'close', 'sense', 'reason', 'ace'), None, <class 'iteratorproxy4xi.IteratorProxy4Xi'>),
            # 'BoundedSemaphore4Xi': (<function BoundedSemaphore at 0x02AD16A8>, ('acquire', 'release', 'sense', 'reason', 'ace'), None, <class 'acquirerproxy4xi.AcquirerProxy4Xi'>),
            #            'Array4Xi': (<function Array4Xi at 0x03043F18>, ('__len__', '__getitem__', '__setitem__'), None, <class 'arrayproxy4xi.ArrayProxy4Xi'>), 
            #                'list': (<class 'list'>, ('sense', 'reason', 'ace'), None, <class 'listproxy4xi.ListProxy4Xi'>),
            #            'Queue4Xi': (<class 'queue.Queue'>, None, None, <function AutoProxy4Xi at 0x0322E7C8>), 
            #             'Pool4Xi': (<class 'pool4xi.Pool4Xi'>, ('apply', 'apply_async', 'close', 'imap', 'imap_unordered', 'join', 'map', 'map_async', 'terminate', 'sense', 'reason', 'ace'), {'apply_async': 'AsyncResult', 'imap_unordered': 'Iterator', 'imap': 'Iterator', 'map_async': 'AsyncResult'}, <class 'poolproxy4xi.PoolProxy4Xi'>),
            #             'Lock4Xi': (<built-in function allocate_lock>, ('acquire', 'release', 'sense', 'reason', 'ace'), None, <class 'acquirerproxy4xi.AcquirerProxy4Xi'>), 
            #        'Semaphore4Xi': (<function Semaphore at 0x02AD1390>, ('acquire', 'release', 'sense', 'reason', 'ace'), None, <class 'acquirerproxy4xi.AcquirerProxy4Xi'>), 
            #            'RLock4Xi': (<function RLock at 0x02AD10C0>, ('acquire', 'release', 'sense', 'reason', 'ace'), None, <class 'acquirerproxy4xi.AcquirerProxy4Xi'>), 
            #                'dict': (<class 'dict'>, ('__contains__', '__delitem__', '__getitem__', '__len__', '__setitem__', 'clear', 'copy', 'get', 'has_key', 'items', 'keys', 'pop', 'popitem', 'setdefault', 'update', 'values', 'sense', 'reason', 'ace'), None, <class 'dictproxy4xi.DictProxy4Xi'>) 
            #    'JoinableQueue4Xi': (<class 'queue.Queue'>, None, None, <function AutoProxy4Xi at 0x0322E7C8>), 
            #            'Value4Xi': (<class 'value4xi.Value4Xi'>, ('get', 'set', 'sense', 'reason', 'ace'), None, <class 'valueproxy4xi.ValueProxy4Xi'>), 
            #        'Namespace4Xi': (<class 'namespace4xi.Namespace4Xi'>, ('__getattribute__', '__setattr__', '__delattr__', 'sense', 'reason', 'ace'), None, <class 'namespaceproxy4xi.NamespaceProxy4Xi'>)
            #}

                        
            message2print = "{(Step 4/12d) passing" + "; hub-of-syncmanager2xi-in-args-passing-to-Process4Xi-by-start-of-SyncManager4Xi: " + str(self._address) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            #PX: \\.\pipe\pyc-1648-0-4b3lku
               
            message2print = "{(Step 4/12e) passing" + "; passport-of-syncmanager2xi-in-args-passing-to-Process4Xi-by-start-of-SyncManager4Xi: " + str(self._authkey) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)            
            #PX: b'^KFO\x83b\xe1\xe5\xfc\x98fI\xd7j\x01U\x86\x0e\xc0[\xa1H\xd2+n$\x96eM\xcc\xc6\x08'; time: 10:00:13 03/24/10 FLE Standard Time

            
            message2print = "{(Step 4/12f) passing" + "; force-of-syncmanager2xi-in-args-passing-to-Process4Xi-by-start-of-SyncManager4Xi: " + str(self._serializer) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            #PX: pickle
            
                        
            message2print = "{(Step 4/12g) passing" + "; wave2write-in-args-passing-to-Process4Xi-by-start-of-SyncManager4Xi: " + str(wave2write) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            #PX: <write-only PipeConnection, handle 440>; time: 10:00:13 03/24/10 FLE Standard Time
            
            message2print = "{(Step 4/12h) calling" + "; Process4Xi-calling-by-start-of-SyncManager4Xi: " + str(Process4Xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
            #PX: <class 'process4xi.Process4Xi'>; time: 10:00:13 03/24/10 FLE Standard Time               
                             
            
                                                                         # spawn process which runs a server
                                                                         # you could list here all methods available for self
           
                                  
                                                                         
            self._process = Process4Xi(                                  #       Note: run-method of Process4Xi is NOT called after start as would be done without own run-method as with server
                target=type(self)._run_server,                           #       target function is the method _run_server
                args=(self._registry, self._address, self._authkey,      #       self_address to be solved
                      self._serializer, wave2write),                     #       wave2write handle received from Pipe4Xi
                )
            
            message2print = "{(Step 4/12i) created" + "; process-of-syncmanager2xi-created-by-start-of-SyncManager4Xi: " + str(self._process) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
            #PX: <Process4Xi(Process4Xi-1, initial)>
            
    
            
            # -----------------------------------------------------------------------------------------    
            # (Step 5/12) OK
            # -----------------------------------------------------------------------------------------                 
                   
            ident = ':'.join(str(i) for i in self._process._identity)
            
            message2print = "{(Step 5/12) set" + "; ident-set-by-start-of-SyncManager4Xi to: " + str(ident)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            #PX: 1
            
              
            # -----------------------------------------------------------------------------------------    
            # (Step 6/12) OK
            # -----------------------------------------------------------------------------------------                 
               
            self._process.name = type(self).__name__  + '-' + ident
            
            message2print = "{(Step 6/12) set" + "; process-name-of-syncmanager2xi-set-by-start-of-SyncManager4Xi to: " + str(self._process.name)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            #PX: SyncManager4Xi-1
               
            # note: eventflow goes as follows: 
            # { Process4Xi : name ; name: SyncManager4Xi-1
            #{ SyncManager4Xi : start ; Step 6/12 set self._process.name to: SyncManager4Xi-1
            
            # --------------------------------------------------------------------------------------------------------    
            # (Step 7/12) - NACK4PX - NOTE THAT SELF._PROCESS.RUN() IS NOT CALLED BECAUSE RUN IS IMPLEMENTED OTHERWISE 
            # --------------------------------------------------------------------------------------------------------                 
               
            self._process.start()
            
            message2print = "{(Step 7/12a) performed" + "; process-start-of-syncmanager2xi-performed-by-start-of-SyncManager4Xi: " + str(self._process.start) + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print)             
            #PX: <bound method Process4Xi.start of <Process4Xi(SyncManager4Xi-3, stopped)>>
            
            omega2sleep4xi(class2print, method2print)         #dragonized giving time to others
            
            ack2kb = False
            nack2kb = True
             
            try:
                kb2process = self._process.sense()
                
                message2print = "{(Step 7/12b) sensed" + "; process-sensed-by-start-of-SyncManager4Xi: " + str(kb2process) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'args-of-process2xi': ({'AsyncResult': (None, None, None, <function AutoProxy4Xi at 0x02F3CED0>),
                #                         'Condition4Xi': (<function Condition at 0x02877738>, ('wait', 'notify', 'notify_all', 'sense', 'reason', 'ace'), None, <class 'conditionproxy4xi.ConditionProxy4Xi'>), 
                #                             'Event4Xi': (<function Event at 0x02877E40>, ('is_set', 'set', 'clear', 'wait', 'sense', 'reason', 'ace'), None, <class 'eventproxy4xi.EventProxy4Xi'>), 
                #                             'Iterator': (None, ('send', 'throw', 'close', 'sense', 'reason', 'ace'), None, <class 'iteratorproxy4xi.IteratorProxy4Xi'>), 
                #                  'BoundedSemaphore4Xi': (<function BoundedSemaphore at 0x02877CD8>, ('acquire', 'release', 'sense', 'reason', 'ace'), None, <class 'acquirerproxy4xi.AcquirerProxy4Xi'>), 
                #                             'Array4Xi': (<function Array4Xi at 0x02F3C8A0>, ('__len__', '__getitem__', '__setitem__'), None, <class 'arrayproxy4xi.ArrayProxy4Xi'>), 
                #                                 'list': (<class 'list'>, ('sense', 'reason', 'ace'), None, <class 'listproxy4xi.ListProxy4Xi'>),
                #                             'Queue4Xi': (<class 'queue.Queue'>, None, None, <function AutoProxy4Xi at 0x02F3CED0>), 
                #                              'Pool4Xi': (<class 'pool4xi.Pool4Xi'>, ('apply', 'apply_async', 'close', 'imap', 'imap_unordered', 'join', 'map', 'map_async', 'terminate', 'sense', 'reason', 'ace'),
                #                         {'apply_async': 'AsyncResult', 'imap_unordered': 'Iterator', 'imap': 'Iterator', 'map_async': 'AsyncResult'}, <class 'poolproxy4xi.PoolProxy4Xi'>), 
                #                              'Lock4Xi': (<built-in function allocate_lock>, ('acquire', 'release', 'sense', 'reason', 'ace'), None, <class 'acquirerproxy4xi.AcquirerProxy4Xi'>), 
                #                         'Semaphore4Xi': (<function Semaphore at 0x028779C0>, ('acquire', 'release', 'sense', 'reason', 'ace'), None, <class 'acquirerproxy4xi.AcquirerProxy4Xi'>), 
                #                             'RLock4Xi': (<function RLock at 0x028776F0>, ('acquire', 'release', 'sense', 'reason', 'ace'), None, <class 'acquirerproxy4xi.AcquirerProxy4Xi'>), 
                #                                 'dict': (<class 'dict'>, ('__contains__', '__delitem__', '__getitem__', '__len__', '__setitem__', 'clear', 'copy', 'get', 'has_key', 'items', 'keys', 'pop', 'popitem', 'setdefault', 'update', 'values', 'sense', 'reason', 'ace'), None, <class 'dictproxy4xi.DictProxy4Xi'>), 
                #                     'JoinableQueue4Xi': (<class 'queue.Queue'>, None, None, <function AutoProxy4Xi at 0x02F3CED0>), 
                #                             'Value4Xi': (<class 'value4xi.Value4Xi'>, ('get', 'set', 'sense', 'reason', 'ace'), None, <class 'valueproxy4xi.ValueProxy4Xi'>), 
                #                         'Namespace4Xi': (<class 'namespace4xi.Namespace4Xi'>, ('__getattribute__', '__setattr__', '__delattr__', 'sense', 'reason', 'ace'), None, <class 'namespaceproxy4xi.NamespaceProxy4Xi'>)}, 
                #                     '\\\\.\\pipe\\pyc-6064-0-epjzog', 
                #                     b'\xa2\xa2x\x90TV\xa95\x88kf\xecTEP\xdfL}*&8O\xe7<K\x00_ \x8b\xaf\xf8\x87',
                #                    'pickle', <write-only PipeConnection, handle 420>),  
                #                    'identity-of-process2xi': (2,), 
                #                       'popen-of-process2xi': <popen4xi.Popen4Xi object at 0x03295EB0>, 
                #                    'daemonic-of-process2xi': False, 'name-of-process2xi': 'SyncManager4Xi-2', 
                #                                    'object': <Process4Xi(SyncManager4Xi-2, stopped)>, 
                #                      'target-of-process2xi': <bound method type._run_server of <class 'syncmanager4xi.SyncManager4Xi'>>, 
                #                     'authkey-of-process2xi': b'\xd9\x1eHM\xe6v\xb0\x8a\x0eV\x8b\xee\x12ca\xc9\xb2n\xf4\x01H]\xab,j\xdb\x83\xc4C\xe0t7', 
                #                      'kwargs-of-process2xi': {},
                #                  'parent_pid-of-process2xi': 6064, 
                #                       'Popen-of-Process4Xi': None,
                #                     'tempdir-of-process2xi': 'c:\\users\\ceo4dr~1\\appdata\\local\\temp\\pymp-8iyqp2'}
                #                                      ; time: 16:32:59 04/06/10 FLE Daylight Time}
                

                
                
                ack2kb = True 
                nack2kb = False    
            except:
                message2print = "{(Step 7/12c) yielded" + "; nack2kb-yielded-by-start-of-SyncManager4Xi: " + str(nack2kb)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 

                  

            # -----------------------------------------------------------------------------------------    
            # (Step 8/12) - OK 
            # -----------------------------------------------------------------------------------------                 
       
            nack2close = False                                #true caused SEVERE PROBLEMS !!!
            if nack2close is True:
                message2print = "{(Step 8/12a) skipped" + "; nack2close-skipped-by-start-of-SyncManager4Xi: " + str(nack2close)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            else:
                wave2write.close()                             #closing is mandatory (tested to be) - this might be because writing to pipe is done by the activated process
            
                message2print = "{(Step 8/12b) performed" + "; wave2write-close-performed-by-start-of-SyncManager4Xi: " + str(wave2write.close) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                #PX: <built-in method close of _multiprocessing.PipeConnection object at 0x02D51EF0>                

            
            # --------------------------------------------------------------------------------------------------------------------    
            # (Step 9/12) TO BE SOLVED: THIS MUST SUCCEED BECAUSE SELF._ADDRESS IS GIVE TO SELF._CLIENT = CLIENT4XI AS PARAMETER !!!
            #             GET ADDRESS OF SERVER
            # --------------------------------------------------------------------------------------------------------------------              
            
            message2print = "{(Step 9/12a) calling" + "; wave2read-recv-calling-by-start-of-SyncManager4Xi: " + str(wave2read.recv) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
            #PX: <built-in method recv of _multiprocessing.PipeConnection object at 0x02D51AA8>              
                                    
            reception_trial_counter2xi = 0
            
            omega2sleep4xi(class2print, method2print)          # dragonized giving time to others
            
            max2count = limit2loop4xi(class2print, method2print) 
            message2print = "{(Step 9/12b) got" + "; max2count-got-by-start-of-SyncManager4Xi: " + str(max2count)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
                       
            while reception_trial_counter2xi < max2count:      # each new trial creates a new temporary file in Pipe4Xi !
            
                reception_trial_counter2xi = reception_trial_counter2xi + 1
                
                message2print = "{(Step 9/12c) set" + "; reception-trial-counter2xi-set-by-start-of-SyncManager4Xi to: " + str(reception_trial_counter2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                #PX: 1
                
                omega2sleep4xi(class2print, method2print)
                
                try: 
                        
                                                                                                        
                    self._address = wave2read.recv()           # SYNCMANAGER2XI-OBJECT ADDRESS = SERVER ADDRESS IS RECEIVED VIA WAVE2READ, SENDER WAVE2WRITE WORKS IN ACTIVATED PROCESS 

                    message2print = "{(Step 9/12d) set" + "; hub-of-syncmanager2xi-set-by-start-of-SyncManager4Xi to: " + str(self._address)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
                    
                    if self._address is None:
                        message2print = "{(Step 9/12e) yielded" + "; wave2read-recv-yielded-by-start-of-SyncManager4Xi: " + str(wave2read.recv)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 

                    else: 
                        message2print = "{(Step 9/12f) got" + "; hub-of-syncmanager2xi-got-by-start-of-SyncManager4Xi: " + str(self._address)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                                    
                       
                        reception_trial_counter2xi = max2count + 33

                   
                        break
                except:
       
             
                    message2print = "{(Step 9/12g) informing" + "; exception-of-reception-informing-by-start-of-SyncManager4Xi: " + " *** SOLVE RECEIVING IN START OF SyncManager4xi, see http://mail.python.org/pipermail/python-list/2009-January/693994.html managers.py " + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                    # PX: to solve receiving in start of SyncManager4xi, see http://mail.python.org/pipermail/python-list/2009-January/693994.html managers.py ; time: 10:00:13 03/24/10 FLE Standard Time}

                                        
                    message2print = "{(Step 9/12h) hit" + "; exception-hit-by-start-of-SyncManager4Xi: " + " *** failed in receiving - reception_trial_counter2xi = " + str(reception_trial_counter2xi)  + " *** " + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # *** failed in receiving - reception_trial_counter2xi = 1 *** ; time: 10:00:13 03/24/10 FLE Standard Time
          
            
            
            # -----------------------------------------------------------------------------------------    
            # (Step 10/12)
            # -----------------------------------------------------------------------------------------              
            message2print = "{(Step 10/1a) calling" + "; wave2read-close-calling-by-start-of-SyncManager4Xi: " + str(wave2read.close) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
     
            try:
                wave2read.close()
             
            except:

                message2print = "{(Step 10/1b) hit" + "; exception-of-close-hit-by-start-of-SyncManager4Xi: " + " *** close function called = " + str(wave2read.close)  + " *** " + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                                    
                                              
                
            # -----------------------------------------------------------------------------------------    
            # (Step 11/12)
            # -----------------------------------------------------------------------------------------                 
       
            # register a finalizer
            self._state.value = State4Xi.STARTED
            
            message2print = "{(Step 11/12) set" + "; state-value-of-syncmanager2xi-set-by-start-of-SyncManager4Xi to: " + str(self._state.value)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                            
                             
            
            # -----------------------------------------------------------------------------------------    
            # (Step 12/12a)
            # -----------------------------------------------------------------------------------------                 
            self.shutdown = Non #dragonized by giving a default value to be tested in _exit
            try:
            
                message2print = "{(Step 12/12a) passing" + "; syncmanager2xi-passing-to-Finalize4Xi-by-start-of-SyncManager4Xi: " + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
            
                message2print = "{(Step 12/12b) passing" + "; type-syncmanager2xi-finalize-manager-passing-to-Finalize4Xi-by-start-of-SyncManager4Xi: " + str(type(self)._finalize_manager) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
            
                message2print = "{(Step 12/12c) passing" + "; 0-passing-to-Finalize4Xi-by-start-of-SyncManager4Xi: " + str(0) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
            
                message2print = "{(Step 12/12d) passing" + "; process-of-syncmanager2xi-in-args-passing-to-Finalize4Xi-by-start-of-SyncManager4Xi: " + str(self._process) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
            
                message2print = "{(Step 12/12e) passing" + "; hub-of-syncmanager2xi-in-args-passing-to-Finalize4Xi-by-start-of-SyncManager4Xi: " + str(self._address) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
            
                message2print = "{(Step 12/12f) passing" + "; passport-of-syncmanager2xi-in-args-passing-to-Finalize4Xi-by-start-of-SyncManager4Xi: " + str(self._authkey) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
            
                message2print = "{(Step 12/12g) passing" + "; state-of-syncmanager2xi-in-args-passing-to-Finalize4Xi-by-start-of-SyncManager4Xi: " + str(self._state) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
            
                message2print = "{(Step 12/12h) passing" + "; Client-of-syncmanager2xi-in-args-passing-to-Finalize4Xi-by-start-of-SyncManager4Xi: " + str(self._Client) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                                                            
                                
                message2print = "{(Step 12/12i) calling" + "; Finalize4Xi-calling-by-start-of-SyncManager4Xi: " + str(Finalize4Xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)       
  
                self.shutdown = Finalize4Xi(self, type(self)._finalize_manager,args=(self._process, self._address, self._authkey,self._state, self._Client),exitpriority=0)
            
            
            except:
                message2print = "{(Step 12/12j) hit" + "; error-hit-by-start-of-SyncManager4Xi: " + " *** class called = " + str(Finalize4Xi)  + " *** " + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                                    
                message2print = "{(Step 12/12k) hit" + "; syncmanager2xi-hit-to-Finalize4Xi-by-start-of-SyncManager4Xi: " + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
            
                message2print = "{(Step 12/12l) hit" + "; type-syncmanager2xi-finalize-manager-hit-to-Finalize4Xi-by-start-of-SyncManager4Xi: " + str(type(self)._finalize_manager) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
            
                message2print = "{(Step 12/12m) hit" + "; 0-hit-to-Finalize4Xi-by-start-of-SyncManager4Xi: " + str(0) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
            
                message2print = "{(Step 12/12n) hit" + "; process-of-syncmanager2xi-in-args-hit-to-Finalize4Xi-by-start-of-SyncManager4Xi: " + str(self._process) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
            
                message2print = "{(Step 12/12o) hit" + "; hub-of-syncmanager2xi-in-args-hit-to-Finalize4Xi-by-start-of-SyncManager4Xi: " + str(self._address) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
            
                message2print = "{(Step 12/12p) hit" + "; passport-of-syncmanager2xi-in-args-hit-to-Finalize4Xi-by-start-of-SyncManager4Xi: " + str(self._authkey) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
            
                message2print = "{(Step 12/12q) hit" + "; state-of-syncmanager2xi-in-args-hit-to-Finalize4Xi-by-start-of-SyncManager4Xi: " + str(self._state) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
            
                message2print = "{(Step 12/12r) hit" + "; Client-of-syncmanager2xi-in-args-hit-to-Finalize4Xi-by-start-of-SyncManager4Xi: " + str(self._Client) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                                                
            
                # { SyncManager4Xi : start ; Step 12/12b exception: *** error: when called Finalize4Xi ***; time: 16:49:30 06/23/09 FLE Daylight Time}          
            
              
            # -----------------------------------------------------------------------------------------    
            # (Step 12/12b)
            # -----------------------------------------------------------------------------------------                    
            
            message2print = "{(Step 12/12b) returning" + "; start-of-SyncManager4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                                    
    #</methodize4xi>
                                
                                    
            
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "run-server-of-SyncManager4Xi" usage = "creates a server, report its address and run it"> 
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>                                 
    # ---------------------------------------------------------------------------------------------
    @classmethod
    def _run_server(cls, registry, address, authkey, force, writer):
        '''
        CREATES A SERVER, REPORT ITS ADDRESS AND RUN IT
        '''

        
        # -----------------------------------------------------------------------------------------    
        #(Step 1/5)
        # -----------------------------------------------------------------------------------------                 
   
  
        class2print = "SyncManager4Xi"
        method2print = "_run_server"
        
        message2print = "{(Step 1/5a) started" + "; run-server-of-SyncManager4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
        message2print = "{(Step 1/5b) passed" + "; class-passed-to-run-server: " + str(cls) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                  
        
        message2print = "{(Step 1/5c) passed" + "; registry-passed-to-run-server: " + str(registry) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                  
        
        message2print = "{(Step 1/5d) passed" + "; address-of-caller-passed-to-run-server: " + str(address) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                  
        
        message2print = "{(Step 1/5e) passed" + "; authkey-passed-to-run-server: " + str(authkey) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                  
        
        message2print = "{(Step 1/5f) passed" + "; force-passed-to-run-server: " + str(force) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                  
        
        message2print = "{(Step 1/5g) passed" + "; writer-passed-to-run-server: " + str(writer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                  
        
        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 
            message2print = "{(StepOriginal) calling" + "; cls-Server-calling-by-run-server-of-SyncManager4Xi: " + str(cls._Server) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
                    
            # create server
            server = cls._Server(registry, address, authkey, force)
            
            message2print = "{(StepOriginal) created" + "; server-created-by-run-server-of-SyncManager4Xi: " + str(server) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
            

            # inform parent process of the server's address
            message2print = "{(StepOriginal) passing" + "; server-address-passing-to-writer-send-by-run-server-of-SyncManager4Xi: " + str(server.address) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            
            writer.send(server.address)
            
            message2print = "{(StepOriginal) performed" + "; writer-send-performed-by-run-server-of-SyncManager4Xi: " + str(writer.send) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             

            writer.close()
            
            message2print = "{(StepOriginal) performed" + "; writer-close-performed-by-run-server-of-SyncManager4Xi: " + str(writer.close) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            # run the manager
            
            message2print = "{(StepOriginal) running" + "; server-serve-forever-running-by-run-server-of-SyncManager4Xi: " + str(server.serve_forever)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                     
                            
            
            server.serve_forever()   
        else:
                  
            # -----------------------------------------------------------------------------------------    
            # (Step 2/5) 
            # -----------------------------------------------------------------------------------------                 
            message2print = "{(Step 2/5a) passing" + "; registry-passing-to-cls-Server-by-run-server-of-SyncManager4Xi: " + str(registry) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                            
            message2print = "{(Step 2/5b) passing" + "; address-of-caller-passing-to-cls-Server-by-run-server-of-SyncManager4Xi: " + str(address) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                                
            message2print = "{(Step 2/5c) passing" + "; authkey-passing-to-cls-Server-by-run-server-of-SyncManager4Xi: " + str(authkey) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            
            message2print = "{(Step 2/5d) passing" + "; force-passing-to-cls-Server-by-run-server-of-SyncManager4Xi: " + str(force) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                    
            ack2syncserver = False
            nack2syncserver = True
            
            try:
            
                server2xi = cls._Server(registry, address, authkey, force)
            
                message2print = "{(Step 2/5e) created" + "; server2xi-created-by-run-server-of-SyncManager4Xi: " + str(server2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
            
                nack2syncserver = False
                ack2syncserver = True
                
            except:
                message2print = "{(Step 2/5f) yielded" + "; nack2syncserver-yielded-by-run-server-of-SyncManager4Xi: " + str(nack2syncserver)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
            
                
            
            # -----------------------------------------------------------------------------------------    
            # (Step 3/5)
            # -----------------------------------------------------------------------------------------                 
            ack2send = False
            nack2send = True
            
            ack2close = False
            nack2close = True  
            
            ack2forever = False
            nack2forever = True                           
            
            if ack2syncserver is True:
            
                message2print = "{(Step 3/5a) passing" + "; server2xi.address-passing-to-writer-send-by-run-server-of-SyncManager4Xi: " + str(server2xi.address) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                                
                try:                 
              
                    writer.send(server2xi.address)
                
                    message2print = "{(Step 3/5b) performed" + "; writer-send-performed-by-run-server-of-SyncManager4Xi: " + str(writer.send) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)             
                    
                    nack2send = False
                    ack2send = True        
                except:
                    message2print = "{yielded" + "; nack2send-yielded-by-run-server-of-SyncManager4Xi: " + str(nack2send)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 

                try:
                    writer.close()
                    
                    nack2close = False
                    ack2close = True  
                    
                    message2print = "{(Step 3/5c) performed" + "; writer-close-performed-by-run-server-of-SyncManager4Xi: " + str(writer.close) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)             
                except:
                    message2print = "{yielded" + "; nack2close-yielded-by-run-server-of-SyncManager4Xi: " + str(nack2close)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 

            
                # -----------------------------------------------------------------------------------------    
                # (Step 4/5)
                # -----------------------------------------------------------------------------------------                 
           
                try:
                    # manager serving
            
                    message2print = "{(Step 4/5) running" + "; server2xi-serve-forever-running-by-run-server-of-SyncManager4Xi: " + str(server2xi.serve_forever)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                                     
                                                 
                            
                    server2xi.serve_forever()
                    
                except:
                    message2print = "{yielded" + "; nack2forever-yielded-by-run-server-of-SyncManager4Xi: " + str(nack2forever)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
                   
            else:
                message2print = "{yielded" + "; nack2forever-yielded-by-run-server-of-SyncManager4Xi: " + str(nack2forever)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                
    
                        
            # -----------------------------------------------------------------------------------------    
            # (Step 5/5)
            # -----------------------------------------------------------------------------------------                 
            message2print = "{(Step 5/5) returning" + "; run-server-of-SyncManager4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                                    
    #</methodize4xi>
                                
                                    
        
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "create-of-SyncManager4Xi" usage = "creates a new shared object; return the token and exposed tuple">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>                              
    # ---------------------------------------------------------------------------------------------
    def _create(self, typeid, *args, **kwds):
        '''
        creates a new shared object; return the token and exposed tuple
        '''

      
        # -----------------------------------------------------------------------------------------    
        # (Step 1/5) OK
        # -----------------------------------------------------------------------------------------         
        class2print = "SyncManager4Xi"
        method2print = "_create"
        
        message2print = "{(Step 1/5a) started" + "; create-of-SyncManager4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        #ALL
        
        message2print = "{(Step 1/5b) passed" + "; typeid-passed-to-create-of-SyncManager4Xi: " + str(typeid) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                  
        #ALL: dict
        
        message2print = "{(Step 1/5c) passed" + "; args-passed-to-create-of-SyncManager4Xi: " + str() + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                  
        #ALL 

        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 
            message2print = "{(StepOriginal calling" + "; SyncManager._create-calling-by-create-of-SyncManager4Xi: " + str(SyncManager._create) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
      
            return SyncManager._create(self, typeid, *args, **kwds)
                 
        else:
                  
                 
            # -----------------------------------------------------------------------------------------    
            # (Step 2/5a) OK
            # -----------------------------------------------------------------------------------------                 
            
            started2xi = SyncManager4Xi._managersc2xi.get_started() # INITIAL = 0, STARTED = 1, SHUTDOWN = 2
 
            message2print = "{(Step 2/5a) set" + "; started2xi-set-by-create-of-SyncManager4Xi to: " + str(started2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            #PX: 1 .i.e. recognized that SyncManager has been started 
            
            # -----------------------------------------------------------------------------------------    
            # (Step 2/5b) OK 
            # -----------------------------------------------------------------------------------------                 
            message2print = "{(Step 2/5b) got" + "; state-value-of-syncmanager2xi-got-by-create-of-SyncManager4Xi: " + str(self._state.value)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #PX: 1
            
            # -----------------------------------------------------------------------------------------    
            # (Step 2/5c) OK 
            # -----------------------------------------------------------------------------------------        
            assert self._state.value == State4Xi.STARTED, 'server not yet started'
            
            message2print = "{(Step 2/5c) asserted" + "; state-value-of-syncmanager2xi-asserted-by-create-of-SyncManager4Xi: " + str(self._state.value) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            #PX: 1                

            
            # -----------------------------------------------------------------------------------------    
            # (Step 3/5)
            # ----------------------------------------------------------------------------------------- 
                            
            message2print = "{(Step 3/5a) passing" + "; hub-of-syncmanager2xi-passing-to-Client-of-syncmanager2xi-by-create-of-SyncManager4Xi: " + str(self._address) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            
                        
            if self._address is None:
                message2print = "{(Step 3/5b) yielded" + "; hub-of-syncmanager2xi-yielded-by-create-of-SyncManager4Xi: " + str(self._address)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
                SyncManager4Xi._ack2server = False 
            else:
                SyncManager4Xi._ack2server = True
                
                message2print = "{(Step 3/5c) informing" + "; hub-of-syncmanager2xi-informing-by-create-of-SyncManager4Xi: " + str(self._address) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                  
                            
            message2print = "{(Step 3/5d) passing" + "; passport-of-syncmanager2xi-passing-to-Client-of-syncmanager2xi-by-create-of-SyncManager4Xi: " + str(self._authkey) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                         
            
        
            # -----------------------------------------------------------------------------------------    
            # 
            # ----------------------------------------------------------------------------------------- 
                                
            if SyncManager4Xi._ack2server is True:                
                
                message2print = "{(Step 3/5e) calling" + "; Client-of-syncmanager2xi-calling-by-create-of-SyncManager4Xi: " + str(self._Client) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)       
                          
                connection2xi = self._Client(self._address, SyncManager4Xi._familyc2xi.get_default_family(), self._authkey)
                
                message2print = "{(Step 3/5f) created" + "; connection2xi-created-by-create-of-SyncManager4Xi: " + str(connection2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                 
                SyncManager4Xi._ack2connection = True 
                        
            else:
                message2print = "{(Step 3/5g) yielded" + "; SyncManager4Xi.ack2connection-yielded-by-create-of-SyncManager4Xi: " + str(" *** ") + str(SyncManager4Xi._ack2connection) + str(" *** ") + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                
                SyncManager4Xi._ack2connection = False
                                 
                #PX: None *** FATAL ERROR to be solved 
            
            
            # -----------------------------------------------------------------------------------------    
            # (Step 4/5)
            # -----------------------------------------------------------------------------------------                 
            if SyncManager4Xi._ack2connection is True:
               
                try:
                    message2print = "{(Step 4/5a) passing" + "; connection2xi-passing-to-dispatch4xi-by-create-of-SyncManager4Xi: " + str(connection2xi) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)            
                            
                    message2print = "{(Step 4/5b) passing" + "; create-manager4xi-passing-to-dispatch4xi-by-create-of-SyncManager4Xi: " + str(SyncManager4Xi._managersc2xi.get_create_manager()) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                                       
                    
                    id2xi, exposed2xi = dispatch4xi(connection2xi, None, SyncManager4Xi._managersc2xi.get_create_manager(), (typeid,)+args, kwds) #CREATE_MANAGER4XI
                
                    message2print = "{(Step 4/5c) set" + "; id2xi-set-by-create-of-SyncManager4Xi to: " + str(id2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
                                    
                
                    message2print = "{(Step 4/5d) set" + "; exposed2xi-set-by-create-of-SyncManager4Xi to: " + str(exposed2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
                                           
                
                except:
                    message2print = "{(Step 4/5e) hit" + "; dispatch4xi-hit-by-create-of-SyncManager4Xi: " + " *** dispatch failed, function =  " + str(dispatch4xi)  + " *** " + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    
                    message2print = "{(Step 4/5f) hit" + "; connection2xi-hit-to-dispatch4xi-by-create-of-SyncManager4Xi: " + " *** dispatch failed, connection =  " + str(connection2xi)  + " *** " +  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)            
                            
                    message2print = "{(Step 4/5g) hit" + "; create-manager4xi-hit-to-dispatch4xi-by-create-of-SyncManager4Xi: " + " *** dispatch failed, create-manager4xi =  " + str(get_create_manager4xi())  + " *** " +  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                                 
                                        
                    
                finally:
    
                    connection2xi.close()
                    
                    message2print = "{(Step 4/5h) performed" + "; connection2xi-close-performed-by-create-of-SyncManager4Xi: " + str(connection2xi.close) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)             
            else:
                message2print = "{(Step 4/5i) yielded" + "; dispatch4xi-yielded-by-create-of-SyncManager4Xi: " + str(dispatch4xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                     
                    
            # -----------------------------------------------------------------------------------------    
            # (Step 5/5)
            # -----------------------------------------------------------------------------------------                 
            if SyncManager4Xi._ack2connection is True:
                message2print = "{(Step 5/5a) passing" + "; typeid-passing-to-Token4Xi-by-create-of-SyncManager4Xi: " + str(typeid) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                
                message2print = "{(Step 5/5b) passing" + "; hub-of-syncmanager2xi-passing-to-Token4Xi-by-create-of-SyncManager4Xi: " + str(self._address) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                             
                
                message2print = "{(Step 5/5c) passing" + "; id-passing-to-Token4Xi-by-create-of-SyncManager4Xi: " + str(id) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                             
    
                message2print = "{(Step 5/5d) returning" + "; Token4Xi-class-returning-by-create-of-SyncManager4Xi: " + str(Token4Xi) + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)  
    
                return Token4Xi(typeid, self._address, id), exposed

            else:
                message2print = "{yielded" + "; Token4Xi-yielded-by-create-of-SyncManager4Xi: " + str(Token4Xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 

            
    #</methodize4xi>
                        
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "join-of-SyncManager4Xi" usage = "joins the manager process (if it has been spawned)">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>                               
    # ---------------------------------------------------------------------------------------------
    def join(self, timeout=None):
        '''
        joins the manager process (if it has been spawned)
        '''

        
        # -----------------------------------------------------------------------------------------    
        # (Step 1/3)
        # -----------------------------------------------------------------------------------------                 
        class2print = "SyncManager4Xi"
        method2print = "join"
        
        message2print = "{(Step 1/3a) started" + "; join-of-SyncManager4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
        timeout = omega2timeout4xi(class2print, method2print) # dragonized 

        message2print = "{(Step 1/3b) set" + "; timeout-set-by-join-of-SyncManager4Xi to: " + str(timeout)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                  

        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 
            message2print = "{(StepOriginal) calling" + "; SyncManager-join-calling-by-join-of-SyncManager4Xi: " + str(SyncManager.join) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
       
            SyncManager.join(self, timeout)
                 
        else:
                
            # -----------------------------------------------------------------------------------------    
            # (Step 2/3)
            # -----------------------------------------------------------------------------------------                 
            message2print = "{(Step 2/3a) passing" + "; timeout-passing-to-process-join-of-syncmanager2xi-by-join-of-SyncManager4Xi: " + str(timeout) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
 
            try:
                self._process.join(timeout)
                
                message2print = "{(Step 2/3b) performed" + "; process-join-of-syncmanager2xi-performed-by-join-of-SyncManager4Xi: " + str(self._process.join) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
            except:
                message2print = "{(Step 2/3c) yielded" + "; process-join-of-syncmanager2xi-yielded-by-join-of-SyncManager4Xi: " + str(self._process.join)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                    
            
            
            # -----------------------------------------------------------------------------------------    
            # (Step 3/3)
            # -----------------------------------------------------------------------------------------                 
            message2print = "{(Step 3/3) returning" + "; join-of-SyncManager4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
    #</methodize4xi>
                    
                                      
        

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "debug-info-of-SyncManager4Xi" usage = "returns some info about the servers shared objects and connections">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>                                   
    # ---------------------------------------------------------------------------------------------
    def _debug_info(self):
        '''
        SyncManager4Xi method _debug_info: returns some info about the servers shared objects and connections
        '''

        # -----------------------------------------------------------------------------------------    
        # (Step 1/4)
        # -----------------------------------------------------------------------------------------          
        class2print = "SyncManager4Xi"
        method2print = "_debug_info"
        
        message2print = "{(Step 1/4a) started" + "; debug-info-of-SyncManager4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
 
        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 
            message2print = "{(StepOriginal) calling" + "; SyncManager-debug-info-calling-by-debug-info-of-SyncManager4Xi: " + str(SyncManager._debug_info) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
            
            return SyncManager._debug_info(self)       
        
        else:
        
            # -----------------------------------------------------------------------------------------    
            # (Step 2/4)
            # -----------------------------------------------------------------------------------------                 
            message2print = "{(Step 2/4a) passing" + "; address-passing-to-Client-of-syncmanager2xi-by-debug-info-of-SyncManager4Xi: " + str(self._address) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                
            message2print = "{(Step 2/4b) passing" + "; authkey-passing-to-Client-of-syncmanager2xi-by-debug-info-of-SyncManager4Xi: " + str(self._authkey) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                               
               
            connection2xi = self._Client(self._address, self._authkey)
            
            message2print = "{(Step 2/4c) set" + "; connection2xi-set-by-debug-info-of-SyncManager4Xi to: " + str(connection2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                
                            
            
            # -----------------------------------------------------------------------------------------    
            # (Step 3/4)
            # -----------------------------------------------------------------------------------------                 
               
            try:
                message2print = "{(Step 3/4a) passing" + "; connection2xi-passing-to-dispatch4xi-by-debug-info-of-SyncManager4Xi: " + str(connection2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                
                message2print = "{(Step 3/4b) passing" + "; debug-info-manager4xi-passing-to-dispatch4xi-by-debug-info-of-SyncManager4Xi: " + str(get_debug_info_manager4xi()) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
     
                return dispatch4xi(connection2xi, None, get_debug_info_manager4xi()) #
            
                
            finally:
                connection2xi.close() 
                
                message2print = "{(Step 3/4c) performed" + "; connection2xi-close-performed-by-debug-info-of-SyncManager4Xi: " + str(connection2xi.close) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                                
                       
            
            # -----------------------------------------------------------------------------------------    
            # (Step 4/4)
            # -----------------------------------------------------------------------------------------                 
            message2print = "{(Step 4/4) returning" + "; debug-info-of-SyncManager4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
    #</methodize4xi>
                    
                                     

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "number-of-objects-of-SyncManager4Xi" usage = "returns the number of shared objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>                                 
    # ---------------------------------------------------------------------------------------------
    def _number_of_objects(self):
        '''
        returns the number of shared objects
        '''

        
        # -----------------------------------------------------------------------------------------    
        # (Step 1/4)
        # -----------------------------------------------------------------------------------------                 
         
        class2print = "SyncManager4Xi"
        method2print = "_number_of_objects"
        
        message2print = "{(Step 1/4a) started" + "; number-of-objects-of-SyncManager4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   

        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 
            message2print = "{(StepOriginal) calling" + "; SyncManager-number-of-objects-calling-by-number-of-objects-of-SyncManager4Xi: " + str(SyncManager._number_of_objects) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
                
                       
            return SyncManager._number_of_objects(self)       
        
        else:
                    
            # -----------------------------------------------------------------------------------------    
            # (Step 2/4)
            # -----------------------------------------------------------------------------------------                 
            message2print = "{(Step 2/4a) passing" + "; hub-of-syncmanager2xi-passing-to-Client-of-syncmanager2xi-by-number-of-objects-of-SyncManager4Xi: " + str(self._address) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            
            message2print = "{(Step 2/4b) passing" + "; passport-of-syncmanager2xi-passing-to-Client-of-syncmanager2xi-by-number-of-objects-of-SyncManager4Xi: " + str(self._authkey) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            
            message2print = "{(Step 2/4c) calling" + "; Client-of-syncmanager2xi-calling-by-number-of-objects-of-SyncManager4Xi: " + str(self._Client) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
                            
                            
             
            connection2xi = self._Client(self._address, self._authkey)
            
            message2print = "{(Step 2/4d) created" + "; connection2xi-created-by-number-of-objects-of-SyncManager4Xi: " + str(connection2xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
           
                    
            # -----------------------------------------------------------------------------------------    
            # (Step 3/4)
            # -----------------------------------------------------------------------------------------                 
               
            try:
                message2print = "{(Step 3/4a) passing" + "; conn-passing-to-dispatch4xi-by-number-of-objects-of-SyncManager4Xi: " + str(conn) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                
                message2print = "{(Step 3/4b) passing" + "; number-of-objects-manager4xi-passing-to-dispatch4xi-by-number-of-objects-of-SyncManager4Xi: " + str(SyncManager4Xi._managersc2xi.get_number_of_objects()) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                     
            
                return dispatch4xi(connection2xi, None, SyncManager4Xi._managersc2xi.get_number_of_objects()) #dragonized NUMBER_OF_OBJECTS_MANAGER4XI
            except:
                message2print = "{(Step 3/4c) yielded" + "; dispatch4xi-yielded-by-number-of-objects-of-SyncManager4Xi: " + str(dispatch4xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 

                    
            finally:
                connection2xi.close()
                
                message2print = "{(Step 3/4c) performed" + "; connection2xi-close-performed-by-number-of-objects-of-SyncManager4Xi: " + str(connection2xi.close) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                        
                        
                        
            # -----------------------------------------------------------------------------------------    
            # (Step 4/4)
            # -----------------------------------------------------------------------------------------                 
            message2print = "{(Step 4/4) returning" + "; number-of-objects-of-SyncManager4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
    #</methodize4xi>
                    
                                       

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "enter-of-SyncManager4Xi" usage = "enters"> 
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>                               
    # ---------------------------------------------------------------------------------------------
    def __enter__(self):
        '''
        enters
        '''

        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
        class2print = "SyncManager4Xi"
        method2print = "__enter__"
        
        message2print = "{started" + "; enter-of-SyncManager4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
   
   
        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 
            message2print = "{(StepOriginal) calling" + "; SyncManager-enter-calling-by-enter-of-SyncManager4Xi: " + str(SyncManager.__enter__) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
                
                        
            return SyncManager.__enter__(self)         
        else:
        
            # -----------------------------------------------------------------------------------------    
            #
            # -----------------------------------------------------------------------------------------                 
            message2print = "{returning" + "; syncmanager2xi-of-SyncManager4Xi-returning-by-enter-of-SyncManager4Xi: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
                                       
            
            return self           
    #</methodize4xi>
            

        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "exit-of-SyncManager4Xi" usage = "exits">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>                                 
    # ---------------------------------------------------------------------------------------------
    def __exit__(self, exc_type, exc_val, exc_tb):
        '''
        exits
        '''

        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 

        class2print = "SyncManager4Xi"
        method2print = "__exit__"
        
        message2print = "{started" + "; exit-of-SyncManager4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
        message2print = "{passed" + "; exc-type-passed-to-exit-of-SyncManager4Xi: " + str(exc_type) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                  
        
        message2print = "{passed" + "; exc-val-passed-to-exit-of-SyncManager4Xi: " + str(exc_val) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                  
        
        message2print = "{passed" + "; exc-tb-passed-to-exit-of-SyncManager4Xi: " + str(exc_tb) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                  
            
    
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
      
        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 
            message2print = "{(StepOriginal) calling" + "; SyncManager-exit-calling-by-exit-of-SyncManager4Xi: " + str(SyncManager.__exit__) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
                
                      
            SyncManager.__exit__(self, exc_type, exc_val, exc_tb)
                    
        else:
            # -----------------------------------------------------------------------------------------
            #     
            # -----------------------------------------------------------------------------------------                 
            message2print = "{calling" + "; shutdown-of-syncmanager2xi-calling-by-exit-of-SyncManager4Xi: " + str(self.shutdown) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
                
            if self.shutdown:   #dragonized
                try:                
                
                    self.shutdown()     
                
                    message2print = "{performed" + "; shutdown-of-syncmanager2xi-performed-by-exit-of-SyncManager4Xi: " + str(self.shutdown) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    
                    SyncManager4Xi._ack2shutdown =  SyncManager4Xi._ack2shutdown + 1
                                                 
                except:
                    SyncManager4Xi._nack2shutdown =  SyncManager4Xi._nack2shutdown + 1
                    
                    message2print = "{yielded" + "; nack2shutdown-yielded-by-exit-of-SyncManager4Xi: " + str(SyncManager4Xi._nack2shutdown)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
            else:
                message2print = "{yielded" + "; shutdown-yielded-by-exit-of-SyncManager4Xi: " + str(self.shutdown)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
               
                           
            message2print = "{returning" + "; exit-of-SyncManager4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
    #</methodize4xi>
                    
                                    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "finalize-manager-of-SyncManager4X" usage = "
    #    shuts down the manager process; will be registered as a finalizer">
    #    _finalize_manager
    #         start(self) <--- funtion in this module
    #         self.shutdown = util.Finalize( <--- makes sure util module is used
    #             self, type(self)._finalize_manager, <--- here 
    #             args=(self._process, self._address, self._authkey,
    #                   self._state, self._Client),
    #             exitpriority=0
    #             )
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>      
    # ---------------------------------------------------------------------------------------------
    @staticmethod
    def _finalize_manager(process, address, authkey, state, _Client):
        '''
        SyncManager4Xi method _finalize_manager: shuts down the manager process; will be registered as a finalizer
        '''

        
        # -----------------------------------------------------------------------------------------    
        # (Step 1/15)
        # -----------------------------------------------------------------------------------------                 

        class2print = "SyncManager4Xi"
        method2print = "_finalize_manager"
        
        message2print = "{(Step 1/15a) started" + "; finalize-manager-of-SyncManager4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
         
        
        message2print = "{(Step 1/15b) passed" + "; process-passed-to-finalize-manager-of-SyncManager4Xi: " + str(process) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                  
           
        
        message2print = "{(Step 1/15c) passed" + "; address-passed-to-finalize-manager-of-SyncManager4Xi: " + str(address) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                  
        
        message2print = "{(Step 1/15d) passed" + "; authkey-passed-to-finalize-manager-of-SyncManager4Xi: " + str(authkey) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                  
        
        message2print = "{(Step 1/15e) passed" + "; state-passed-to-finalize-manager-of-SyncManager4Xi: " + str(state) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                  
        
        message2print = "{(Step 1/15f) passed" + "; Client-passed-to-finalize-manager-of-SyncManager4Xi: " + str(_Client) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                  
                 
       
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------

        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 
            message2print = "{(StepOriginal) calling" + "; SyncManager-finalize-manager-calling-by-finalize-manager-of-SyncManager4Xi: " + str(SyncManager._finalize_manager) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
                
                      
            SyncManager._finalize_manager(process, address, authkey, state, _Client)
                    
        else:                
        
        
            # -----------------------------------------------------------------------------------------    
            # (Step 2/15)
            # -----------------------------------------------------------------------------------------                 
               
            if process.is_alive():
                # sending shutdown message to manager
                
                message2print = "{(Step 2/15a) decided" + "; shutdown-message-decided-by-finalize-manager-of-SyncManager4Xi: " + "function used for decision = " + str(process.is_alive)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                   
                
                try:
        
                    # ---------------------------------------------------------------------------------    
                    # (Step 3/15)
                    # ---------------------------------------------------------------------------------                 
                    message2print = "{(Step 3/15a) passing" + "; address-passing-to-Client-by-finalize-manager-of-SyncManager4Xi: " + str(address) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)            
                
                    message2print = "{(Step 3/15b) passing" + "; authkey-passing-to-Client-by-finalize-manager-of-SyncManager4Xi: " + str(authkey) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)            
                                      
                    
                    
                    connection2xi = _Client(address, authkey)
                    
                    message2print = "{{(Step 3/15c) set" + "; connection2xi-set-by-finalize-manager-of-SyncManager4Xi to: " + str(connection2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
                
                           
                    
        
                    # ---------------------------------------------------------------------------------    
                    # (Step 4/15)
                    # ---------------------------------------------------------------------------------                 
                               
                    try:
                        message2print = "{(Step 4/15a) passing" + "; connection2xi-passing-to-dispatch4xi-by-finalize-manager-of-SyncManager4Xi: " + str(connection2xi) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)            
                        
                        message2print = "{(Step 4/15b) passing" + "; None-passing-to-dispatch4xi-by-finalize-manager-of-SyncManager4Xi: " + str(None) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)                 
                        
                        message2print = "{(Step 4/15c) passing" + "; shutdown-passing-to-dispatch4xi-by-finalize-manager-of-SyncManager4Xi: " + str('shutdown') + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)                                     
                    
                        dispatch4xi(connection2xi, None, 'shutdown')
                        
                        message2print = "{(Step 4/15d) performed" + "; dispatch4xi-performed-by-finalize-manager-of-SyncManager4Xi: " + str(dispatch4xi) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)             
                    except:
                        message2print = "{(Step 4/15e) yielded" + "; dispatch4xi-yielded-by-finalize-manager-of-SyncManager4Xi: " + str(dispatch4xi)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
   
        
                    # ---------------------------------------------------------------------------------    
                    # (Step 5/15)
                    # ---------------------------------------------------------------------------------                 
                                   
                    finally:
                        connection2xi.close()
                        
                        message2print = "{(Step 5/15) performed" + "; connection2xi-close-performed-by-finalize-manager-of-SyncManager4Xi: " + str(connection2xi.close) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)             
                                        
      
                                             
                except Exception:
        
                    # ---------------------------------------------------------------------------------    
                    # (Step 6/15)
                    # ---------------------------------------------------------------------------------                 
                    message2print = "{(Step 6/15a) hit" + "; Exception-hit-by-finalize-manager-of-SyncManager4Xi: " + " *** failed to shutdown  *** " + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
        
                    message2print = "{(Step 6/15b) hit" + "; process-hit-to-finalize-manager-of-SyncManager4Xi: " + str(process) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                  
           
        
                    message2print = "{(Step 6/15c) hit" + "; address-hit-to-finalize-manager-of-SyncManager4Xi: " + str(address) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                  
        
                    message2print = "{(Step 6/15d) hit" + "; authkey-hit-to-finalize-manager-of-SyncManager4Xi: " + str(authkey) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                  
        
                    message2print = "{(Step 6/15e) hit" + "; state-hit-to-finalize-manager-of-SyncManager4Xi: " + str(state) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                  
        
                    message2print = "{(Step 6/15f) hit" + "; Client-hit-to-finalize-manager-of-SyncManager4Xi: " + str(_Client) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                  
              
                message2print = "{(Step 6/15h) passing" + "; timeout-passing-to-process-join-by-finalize-manager-of-SyncManager4Xi: " + str(0.2) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            

                timeout = omega2timeout4xi(class2print, method2print) 

                message2print = "{set" + "; timeout-set-by-put-of-Queue4Xi to: " + str(timeout)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                
                process.join(timeout)
                
                message2print = "{(Step 6/15i) performed" + "; process-join-performed-by-finalize-manager-of-SyncManager4Xi: " + str(process.join) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                                  
                
                
                if process.is_alive():
        
                    # ---------------------------------------------------------------------------------    
                    # (Step 7/15)
                    # ---------------------------------------------------------------------------------                 
                   
                    # manager still alive
                    
                    message2print = "{(Step 7/15) informing" + "; manager-alive-informing-by-finalize-manager-of-SyncManager4Xi: " + "manager still alive" + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                      
                    
                    
                    if hasattr(process, 'terminate'):
                        
                        # -----------------------------------------------------------------------------    
                        # (Step 8/15)
                        # -----------------------------------------------------------------------------                 
                       
                        # trying to terminate manager process

                        message2print = "{(Step 8/15a) calling" + "; process-terminate-calling-by-finalize-manager-of-SyncManager4Xi: " + str(process.terminate) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)       
                
                                        
                        process.terminate()
                        
                        message2print = "{(Step 8/15b) performed" + "; process-terminate-performed-by-finalize-manager-of-SyncManager4Xi: " + str(process.terminate) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)             
                                        
                        
                        # -----------------------------------------------------------------------------    
                        # (Step 9/15)
                        # -----------------------------------------------------------------------------                 

                        message2print = "{(Step 9/15a) calling" + "; process-join-calling-by-finalize-manager-of-SyncManager4Xi: " + str(process.join) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)       
                        
                        message2print = "{(Step 9/15b) passing" + "; timeout-passing-to-process-join-by-finalize-manager-of-SyncManager4Xi: " + str(0.1) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)            
                
                            
                        process.join(timeout)
                        
                        message2print = "{(Step 9/15c) performed" + "; process-join-performed-by-finalize-manager-of-SyncManager4Xi: " + str(process.join) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)             
                           
                        # -----------------------------------------------------------------------------    
                        # (Step 10/15)
                        # ----------------------------------------------------------------------------- 
                        message2print = "{(Step 10/15) calling" + "; process-is-alive-calling-by-finalize-manager-of-SyncManager4Xi: " + str(process.is_alive) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)       
                               
                        if process.is_alive():
                                                
                            # -------------------------------------------------------------------------    
                            # (Step 11/15)
                            # -------------------------------------------------------------------------                 
     
                            # manager still alive after terminate !
                            
                            message2print = "{(Step 11/15) informing" + "; alive-informing-by-finalize-manager-of-SyncManager4Xi: " + " manager still alive after terminate ! " + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)  
     
            
            # -----------------------------------------------------------------------------------------    
            # (Step 12/15)
            # -----------------------------------------------------------------------------------------                 
                   
            state.value = State4Xi.SHUTDOWN
            
            message2print = "{(Step 12/15) set" + "; state-value-set-by-finalize-manager-of-SyncManager4Xi to: " + str(state.value)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                
                            
            
            try:
        
                # -------------------------------------------------------------------------------------    
                # (Step 13/15)
                # -------------------------------------------------------------------------------------                 
                message2print = "{(Step 13/15) passing" + "; address-passing-to-del-by-finalize-manager-of-SyncManager4Xi: " + str(address) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                
                               
                del BaseProxy4Xi._address_to_local[address]
            
             
                
            except KeyError:
        
                # -------------------------------------------------------------------------------------    
                # (Step 14/15)
                # -------------------------------------------------------------------------------------                 
                message2print = "{(Step 14/15) hit" + "; KeyError-hit-by-finalize-manager-of-SyncManager4Xi: " + " *** del BaseProxy4Xi._address_to_local[address], where address = " + str(address)  + " *** " + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)

                             
            # -----------------------------------------------------------------------------------------    
            # (Step 15/15)
            # -----------------------------------------------------------------------------------------                 
            message2print = "{(Step 15/15) returning" + "; finalize-manager-of-SyncManager4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
    #</methodize4xi>
                    
                                         
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "register-of-SyncManager4Xi" usage = "
    #    registers a typeid with the manager type and return proxy">
    #    SyncManager4Xi.register('Queue4Xi', queue.Queue)
    #    SyncManager4Xi.register('JoinableQueue4Xi', queue.Queue)
    #    SyncManager4Xi.register('Event4Xi', threading.Event, EventProxy4Xi)
    #    SyncManager4Xi.register('Lock4Xi', threading.Lock, AcquirerProxy4Xi)
    #    SyncManager4Xi.register('RLock4Xi', threading.RLock, AcquirerProxy4Xi)
    #    SyncManager4Xi.register('Semaphore4Xi', threading.Semaphore, AcquirerProxy4Xi)
    #    SyncManager4Xi.register('BoundedSemaphore4Xi', threading.BoundedSemaphore, AcquirerProxy4Xi)
    #    SyncManager4Xi.register('Condition4Xi', threading.Condition, ConditionProxy4Xi)
    #
    #    SyncManager4Xi.register('Pool4Xi', Pool4Xi, PoolProxy4Xi)
    #    SyncManager4Xi.register('list', list, ListProxy4Xi)
    #    SyncManager4Xi.register('dict', dict, DictProxy4Xi)
    #    SyncManager4Xi.register('Value4Xi', Value, ValueProxy4Xi)
    #    SyncManager4Xi.register('Array4Xi', Array, ArrayProxy4Xi)
    #    SyncManager4Xi.register('Namespace4Xi', Namespace, 4Xi)

    # types returned by methods of PoolProxy4Xi
    #    SyncManager4Xi.register('Iterator4Xi', proxytype=IteratorProxy, create_method=False)
    #    SyncManager4Xi.register('AsyncResult4Xi', create_method=False)  
    
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/> 
    # ---------------------------------------------------------------------------------------------
    @classmethod
    def register(cls, typeid, callable=None, proxytype=None, exposed=None,
                 method_to_typeid=None, create_method=True):
        '''
        SyncManager4Xi method register: registers a typeid with the manager type and return proxy
        '''

        
        # -----------------------------------------------------------------------------------------    
        # (Step 1/9)
        # -----------------------------------------------------------------------------------------                 
        
        
        class2print = "SyncManager4Xi"
        method2print = "register"

        
        message2print = "{(Step 1/9a) started" + "; register-of-SyncManager4Xi-started: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
        message2print = "{(Step 1/9b) passed" + "; class-passed-to-register-of-SyncManager4Xi: " + str(cls) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                  
        #<class 'syncmanager4xi.SyncManager4Xi'>
        
        
        message2print = "{(Step 1/9c) passed" + "; typeid-passed-to-register-of-SyncManager4Xi: " + str(typeid) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                  
        #Queue4Xi
        
        message2print = "{(Step 1/9d) passed" + "; callable-passed-to-register-of-SyncManager4Xi: " + str(callable) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                  
        #<class 'queue.Queue'>
        
        message2print = "{(Step 1/9e) passed" + "; proxytype-passed-to-register-of-SyncManager4Xi: " + str(proxytype) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                  
        #None
        
        message2print = "{(Step 1/9f) passed" + "; exposed-passed-to-register-of-SyncManager4Xi: " + str(exposed) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                  
        # None
        
        message2print = "{(Step 1/9g) passed" + "; method-to-typeid-passed-to-register-of-SyncManager4Xi: " + str(method_to_typeid) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                  
        # None
        
        message2print = "{(Step 1/9h) passed" + "; create-method-passed-to-register-of-SyncManager4Xi: " + str(create_method) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                  
        # True                
                
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------
                                        
        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 
            message2print = "{(StepOriginal) calling" + "; SyncManager-register-calling-by-register-of-SyncManager4Xi: " + str(SyncManager.register) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
                
             
            SyncManager.register(cls, typeid, callable, proxytype, exposed, method_to_typeid, create_method)
            
            
        else:                
                
        
        
            # -----------------------------------------------------------------------------------------
            # (Step 2/9)
            #  add _registry to caller's class dictionary, if necessary
            # -----------------------------------------------------------------------------------------
            # continue her; to be done:
            # problem: parameter cls = SyncManager4Xi, however SyncManager4Xi does NOT have attribute _registry!
            #
            if '_registry' not in cls.__dict__:
                cls._registry = cls._registry.copy()
            
                message2print = "{(Step 2/9) set" + "; cls-registry-set-by-register-of-SyncManager4Xi to: " + str(cls._registry)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                
                            

                              
            # -----------------------------------------------------------------------------------------
            # (Step 3/9)
            # set proxytype, if necessary
            # -----------------------------------------------------------------------------------------
            if proxytype is None:
                proxytype = AutoProxy4Xi
                
            
                message2print = "{(Step 3/9) set" + "; proxytype-set-by-register-of-SyncManager4Xi to: " + str(proxytype)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                # <function AutoProxy4Xi at 0x02DCFFA8>
                
    
            # -----------------------------------------------------------------------------------------
            # (Step 4/9)
            # -----------------------------------------------------------------------------------------
            exposed = exposed or getattr(proxytype, '_exposed_', None)
            # None
            
            message2print = "{(Step 4/9) set" + "; exposed-set-by-register-of-SyncManager4Xi to: " + str(exposed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # None  
            
            # -----------------------------------------------------------------------------------------
            # (Step 5/9)
            # -----------------------------------------------------------------------------------------
            method_to_typeid = method_to_typeid or \
                               getattr(proxytype, '_method_to_typeid_', None)
            
            
            message2print = "{(Step 5/9) set" + "; method-to-typeid-set-by-register-of-SyncManager4Xi to: " + str(method_to_typeid)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # None                    
            
            # -----------------------------------------------------------------------------------------
            # (Step 6/9)
            # -----------------------------------------------------------------------------------------
            if method_to_typeid:
                for key, value in list(method_to_typeid.items()):
                    assert type(key) is str, '%r is not a string' % key
                    
                    message2print = "{(Step 6/9a) asserted" + "; key-asserted-by-register-of-SyncManager4Xi: " + str(key) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)             
                    
                    
                    
                    assert type(value) is str, '%r is not a string' % value
                    
                    message2print = "{(Step 6/9a) asserted" + "; value-asserted-by-register-of-SyncManager4Xi: " + str(value) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)             
                      
                    
            
                    

                            
            # -----------------------------------------------------------------------------------------
            # (Step 7/9)
            # -----------------------------------------------------------------------------------------
            cls._registry[typeid] = (callable, exposed, method_to_typeid, proxytype)
    
            
            message2print = "{(Step 7/9a) using" + "; cls-registry-typeid-using-by-register-of-SyncManager4Xi: " + str(typeid)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #01. Queue4Xi
            #02. JoinableQueue4Xi
            #03. Event4Xi
            #04. Lock4Xi
            #05. RLock4Xi
            #06. Semaphore4Xi
            #07. BoundedSemaphore4Xi
            #08. Condition4Xi
            #09. Pool4Xi
            #10. list
            #11. dict
            #12. Value4Xi
            #13. Array4Xi
            #14. Namespace4Xi
            #15. Iterator
            #16. AsyncResult

            
            message2print = "{(Step 7/9b) set" + "; cls-registry-element-set-by-register-of-SyncManager4Xi to: " + str(cls._registry[typeid])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)     
            #01. (<class 'queue.Queue'>, None, None, <function AutoProxy4Xi at 0x02FF3B70>)
            #02. (<class 'queue.Queue'>, None, None, <function AutoProxy4Xi at 0x02FF3B70>)
            #03. (<function Event at 0x02893390>, ('is_set', 'set', 'clear', 'wait', 'sense', 'reason', 'ace'), None, <class 'eventproxy4xi.EventProxy4Xi'>)
            #04. (<built-in function allocate_lock>, ('acquire', 'release', 'sense', 'reason', 'ace'), None, <class 'acquirerproxy4xi.AcquirerProxy4Xi'>)
            #05. (<function RLock at 0x02888C00>, ('acquire', 'release', 'sense', 'reason', 'ace'), None, <class 'acquirerproxy4xi.AcquirerProxy4Xi'>)
            #06. (<function Semaphore at 0x02888ED0>, ('acquire', 'release', 'sense', 'reason', 'ace'), None, <class 'acquirerproxy4xi.AcquirerProxy4Xi'>)
            #07. (<function BoundedSemaphore at 0x02893228>, ('acquire', 'release', 'sense', 'reason', 'ace'), None, <class 'acquirerproxy4xi.AcquirerProxy4Xi'>)
            #08. (<function Condition at 0x02888C48>, ('wait', 'notify', 'notify_all', 'sense', 'reason', 'ace'), None, <class 'conditionproxy4xi.ConditionProxy4Xi'>)
            #09. (<class 'pool4xi.Pool4Xi'>, ('apply', 'apply_async', 'close', 'imap', 'imap_unordered', 'join', 'map', 'map_async', 'terminate', 'sense', 'reason', 'ace'), {'apply_async': 'AsyncResult', 'imap_unordered': 'Iterator', 'imap': 'Iterator', 'map_async': 'AsyncResult'}, <class 'poolproxy4xi.PoolProxy4Xi'>)
            #10. (<class 'list'>, ('sense', 'reason', 'ace'), None, <class 'listproxy4xi.ListProxy4Xi'>)
            #11. (<class 'dict'>, ('__contains__', '__delitem__', '__getitem__', '__len__', '__setitem__', 'clear', 'copy', 'get', 'has_key', 'items', 'keys', 'pop', 'popitem', 'setdefault', 'update', 'values', 'sense', 'reason', 'ace'), None, <class 'dictproxy4xi.DictProxy4Xi'>)
            #12. (<class 'value4xi.Value4Xi'>, ('get', 'set', 'sense', 'reason', 'ace'), None, <class 'valueproxy4xi.ValueProxy4Xi'>)
            #13. (<function Array4Xi at 0x0295FED0>, ('__len__', '__getitem__', '__setitem__'), None, <class 'arrayproxy4xi.ArrayProxy4Xi'>)
            #14. (<class 'namespace4xi.Namespace4Xi'>, ('__getattribute__', '__setattr__', '__delattr__', 'sense', 'reason', 'ace'), None, <class 'namespaceproxy4xi.NamespaceProxy4Xi'>)
            #15. (None, ('send', 'throw', 'close', 'sense', 'reason', 'ace'), None, <class 'iteratorproxy4xi.IteratorProxy4Xi'>)
            #16. (None, None, None, <function AutoProxy4Xi at 0x02FF3B70>)

            

            # -----------------------------------------------------------------------------------------
            # 
            # -----------------------------------------------------------------------------------------
            if create_method:
    
                # -------------------------------------------------------------------------------------
                # (Step 8/9)
                #<function4xi function = "temp" usage = "
                # embedded function inside Syncmmanager4Xi
                #"> 
                #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/> 
                # -------------------------------------------------------------------------------------
                def temp(self, *args, **kwds):
                    # requesting creation of a shared object
                    
                    message2print = "{(Step 8/9a) passing" + "; typeid-passing-to-create-of-syncmanager2xi-by-temp-of-SyncManager4Xi: " + str(typeid) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)            
                    #PX: dict <-- dict2xi = manager2xi.dict() 
                    
                    token, exp = self._create(typeid, *args, **kwds)
            
                    message2print = "{(Step 8/9b) set" + "; token-set-by-temp-of-SyncManager4Xi to: " + str(token)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
                    
                    message2print = "{(Step 8/9c) set" + "; exp-set-by-temp-of-SyncManager4Xi to: " + str(exp)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
                         
                    proxy = proxytype(
                        token, self._serializer, manager=self,
                        authkey=self._authkey, exposed=exp
                        )
                    
                    message2print = "{(Step 8/9d) set" + "; proxy-set-by-temp-of-SyncManager4Xi to: " + str(proxy)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
                       
                    conn = self._Client(token.address, self._authkey)
                    
                    message2print = "{(Step 8/9e) set" + "; conn-set-by-temp-of-SyncManager4Xi to: " + str(conn)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
                       
                    dispatch4xi(conn, None, 'decref', (token.id,))
                    
                    message2print = "{(Step 8/9f) returning" + "; proxy-returning-by-temp-of-SyncManager4Xi: " + str(proxy) + "; time: " + time4xi() + "}" 
                    print4xi(class2print, method2print, message2print)    
                                            
                       
                    return proxy
                #</function4xi>
                 
                    
                temp.__name__ = typeid
                
                    
                message2print = "{(Step 8/9g) set" + "; temp-name-set-by-register-of-SyncManager4Xi to: " + str(temp.__name__)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                   
                message2print = "{(Step 8/9h) passing" + "; class-passing-to-setattr-by-register-of-SyncManager4Xi: " + str(cls) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                   
                message2print = "{(Step 8/9i) passing" + "; typeid-passing-to-setattr-by-register-of-SyncManager4Xi: " + str(typeid) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                   
                message2print = "{(Step 8/9j) passing" + "; temp-passing-to-setattr-by-register-of-SyncManager4Xi: " + str(temp) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                      
                                
                setattr(cls, typeid, temp)
                

            # -------------------------------------------------------------------------------------
            # (Step 9/9)
            # -------------------------------------------------------------------------------------             
            
            message2print = "{(Step 9/9) returning" + "; register-of-SyncManager4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
    #</methodize4xi>
                    
                             
        

                                       
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-SyncManager4Xi" usage = "senses SyncManager4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>     
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses SyncManager4Xi class objects
        '''
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        
        class2print = 'SyncManager4Xi'
        method2print = 'sense'
        
        
        message2print = "{started" + "; sense-of-SyncManager4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
   
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        message2print = "{sensing" + "; SyncManager4Xi-sensing-by-sense-of-SyncManager4Xi: " + str(SyncManager4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                      
        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb2xi = sense_object4xi(self)
        
        message2print = "{got" + "; knowledge-base2xi-got-by-sense-of-SyncManager4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # -----------------------------------------------------------------------------------------         
        try:
            kb2xi['hub-of-syncmanager2xi'] = self._address                                     #OBJECTS VALUES
            kb2xi['passport-of-syncmanager2xi'] = self._authkey        
            kb2xi['state-of-syncmanager2xi'] = self._state
            kb2xi['force-of-syncmanager2xi'] = self._serializer   
            kb2xi['Listener-of-syncmanager2xi'] = self._Listener
            kb2xi['Client-of-syncmanager2xi'] = self._Client  
            
            kb2xi['registry-of-SyncManager4Xi'] = SyncManager4Xi._registry                         #REGISTRY

            kb2xi['Server-of-SyncManager4Xi'] = SyncManager4Xi._Server                             #SERVER

    
            kb2xi['afterfork2xi-of-SyncManager4Xi'] = SyncManager4Xi._afterfork2xi                 #CONSTANTS  
            kb2xi['managersc2xi-of-SyncManager4Xi'] = SyncManager4Xi._managersc2xi  
            kb2xi['utilc2xi-of-SyncManager4Xi'] = SyncManager4Xi._utilc2xi 
            kb2xi['connectionc2xi-of-SyncManager4Xi'] = SyncManager4Xi._connectionc2xi  
            kb2xi['familyc2xi-of-SyncManager4Xi'] = SyncManager4Xi._familyc2xi  
            kb2xi['forkingc2xi-of-SyncManager4Xi'] = SyncManager4Xi._forkingc2xi 
            kb2xi['heapc2xi-of-SyncManager4Xi'] = SyncManager4Xi._heapc2xi  
            kb2xi['loggerc2xi-of-SyncManager4Xi'] = SyncManager4Xi._loggerc2xi  
            kb2xi['poolc2xi-of-SyncManager4Xi'] = SyncManager4Xi._poolc2xi 
            kb2xi['processc2xi-of-SyncManager4Xi'] = SyncManager4Xi._processc2xi  
            kb2xi['synchronizec2xi-of-SyncManager4Xi'] = SyncManager4Xi._synchronizec2xi  
            kb2xi['systemc2xi-of-SyncManager4Xi'] = SyncManager4Xi._systemc2xi 

            kb2xi['ack2connection-of-SyncManager4Xi'] = SyncManager4Xi._ack2connection             #STATES 
            kb2xi['nack2connection-of-SyncManager4Xi'] = SyncManager4Xi._nack2connection 
            
            kb2xi['ack2server-of-SyncManager4Xi'] = SyncManager4Xi._ack2server   
            kb2xi['nack2server-of-SyncManager4Xi'] = SyncManager4Xi._nack2server                         
            
            kb2xi['ack2dispatch-of-SyncManager4Xi'] = SyncManager4Xi._ack2dispatch   
            kb2xi['nack2dispatch-of-SyncManager4Xi'] = SyncManager4Xi._nack2dispatch   
            
            kb2xi['ack2shutdown-of-SyncManager4Xi'] = SyncManager4Xi._ack2shutdown                 #STATISCTICS
            kb2xi['nack2shutdown-of-SyncManager4Xi'] = SyncManager4Xi._nack2shutdown  
            
            kb2xi['ack2type-of-SyncManager4Xi'] = SyncManager4Xi._ack2type
            kb2xi['nack2type-of-SyncManager4Xi'] = SyncManager4Xi._nack2type                                               
        except:
            message2print = "{yielded" + "; knowledge-base2xi-yielded-by-sense-of-SyncManager4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 

 
        message2print = "{sensed" + "; SyncManager4Xi-sensed-by-sense-of-SyncManager4Xi: " + str(SyncManager4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        message2print = "{returning" + "; knowledge-base2xi-returning-by-sense-of-SyncManager4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                                 
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-SyncManager4Xi" usage = "reasons SyncManager4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>     
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons SyncManager4Xi class objects
        '''
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        
        class2print = 'SyncManager4Xi'
        method2print = 'reason'

        
        message2print = "{started" + "; reason-of-SyncManager4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
       
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{reasoning" + "; SyncManager4Xi-reasoning-by-reason-of-SyncManager4Xi: " + str(SyncManager4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
                
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        kb4xi = reason_object4xi(self)
        
        
        message2print = "{performed" + "; knowledge-base4xi-performed-by-reason-of-SyncManager4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
                               
                
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        
        
          
        
        message2print = "{reasoned" + "; SyncManager4Xi-reasoned-by-reason-of-SyncManager4Xi: " + str(SyncManager4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{zeroed" + "; knowledge-base4xi-zeroed-by-reason-of-SyncManager4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
                      
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-SyncManager4Xi" usage = "aces SyncManager4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>     
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces SyncManager4Xi class objects 
        '''
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        
        
        class2print = 'SyncManager4Xi'
        method2print = 'ace'
        
        
        message2print = "{started" + "; ace-of-SyncManager4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
           
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{acing" + "; SyncManager4Xi-acing-by-ace-of-SyncManager4Xi: " + str(SyncManager4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                    
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{acquiring" + "; knowledge-base8xi-acquiring-by-ace-of-SyncManager4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                 
                


        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        
          
        message2print = "{aced" + "; SyncManager4Xi-aced-by-ace-of-SyncManager4Xi: " + str(SyncManager4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{calling" + "; knowledge-base8xi-calling-by-ace-of-SyncManager4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
                
                     
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
#</class4xi> 

# -------------------------------------------------------------------------------------------------
# Registration parameters                             
# ------------------------------------------------------------------------------------------------- 
# 1. cls = class
# 2. typeid
# 3. callable (optional)
# 4. proxytype (optional)
# 5. exposed (optional)
# 6. method_to_typeid (optional)
# 7. create_method (optional)


# If executed here, registration doubles because __init__ does the same registration !
# SyncManager4Xi.register('Queue4Xi', queue.Queue) 
# SyncManager4Xi.register('JoinableQueue4Xi', queue.Queue)
# SyncManager4Xi.register('Event4Xi', threading.Event, EventProxy4Xi)
# SyncManager4Xi.register('Lock4Xi', threading.Lock, AcquirerProxy4Xi)
# SyncManager4Xi.register('RLock4Xi', threading.RLock, AcquirerProxy4Xi)
# SyncManager4Xi.register('Semaphore4Xi', threading.Semaphore, AcquirerProxy4Xi)
# SyncManager4Xi.register('BoundedSemaphore4Xi', threading.BoundedSemaphore, AcquirerProxy4Xi)
# SyncManager4Xi.register('Condition4Xi', threading.Condition, ConditionProxy4Xi)
# SyncManager4Xi.register('Pool4Xi', Pool4Xi, PoolProxy4Xi)
# SyncManager4Xi.register('list', list, ListProxy4Xi)
# SyncManager4Xi.register('dict', dict, DictProxy4Xi)
# SyncManager4Xi.register('Value4Xi', Value4Xi, ValueProxy4Xi)
# SyncManager4Xi.register('Array4Xi', Array4Xi, ArrayProxy4Xi)
# SyncManager4Xi.register('Namespace4Xi', Namespace4Xi, NamespaceProxy4Xi)
# SyncManager4Xi.register('Iterator4Xi', proxytype=IteratorProxy4Xi, create_method=False)
# SyncManager4Xi.register('AsyncResult', create_method=False)                                       #PoolProxy4Xi._method_to_typeid_

#</program4xi>
 
# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = " 
#
#   test_proxy_mp_xi4.py.txt
#   sense-proxy-mp-test4xi.xml
#   reason-proxy-mp-test4xi.xml
#   ace-proxy-mp-test4xi.xml
#
#"
#>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 
#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of syncmanager4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>

